/******************************************************************/
/* cvtypes.h */

typedef unsigned int size_t;
typedef long long int64;
typedef unsigned long long uint64;
typedef unsigned char uchar;

typedef void CvArr;

#define IPL_DEPTH_SIGN 0x80000000

#define IPL_DEPTH_1U     1
#define IPL_DEPTH_8U     8
#define IPL_DEPTH_16U   16
#define IPL_DEPTH_32F   32

#define IPL_DEPTH_8S  (IPL_DEPTH_SIGN| 8)
#define IPL_DEPTH_16S (IPL_DEPTH_SIGN|16)
#define IPL_DEPTH_32S (IPL_DEPTH_SIGN|32)

#define IPL_DATA_ORDER_PIXEL  0
#define IPL_DATA_ORDER_PLANE  1

#define IPL_ORIGIN_TL 0
#define IPL_ORIGIN_BL 1

#define IPL_ALIGN_4BYTES   4
#define IPL_ALIGN_8BYTES   8
#define IPL_ALIGN_16BYTES 16
#define IPL_ALIGN_32BYTES 32

#define IPL_ALIGN_DWORD   IPL_ALIGN_4BYTES
#define IPL_ALIGN_QWORD   IPL_ALIGN_8BYTES

typedef struct _IplImage {
    int  nSize;
    int  ID;
    int  nChannels;
    int  depth;
    int  dataOrder;
    int  origin;
    int  width;
    int  height;
    struct _IplROI *roi;
    int  imageSize;
    char *imageData;
    int  widthStep;
    char *imageDataOrigin;
}
IplImage;

typedef struct _IplTileInfo IplTileInfo;

typedef struct _IplROI {
    int  coi;
    int  xOffset;
    int  yOffset;
    int  width;
    int  height;
}
IplROI;

typedef struct _IplConvKernel
{
    int  nCols;
    int  nRows;
    int  anchorX;
    int  anchorY;
    int *values;
    int  nShiftR;
}
IplConvKernel;

typedef struct _IplConvKernelFP
{
    int  nCols;
    int  nRows;
    int  anchorX;
    int  anchorY;
    float *values;
}
IplConvKernelFP;

#define IPL_IMAGE_HEADER 1
#define IPL_IMAGE_DATA   2
#define IPL_IMAGE_ROI    4

#define IPL_IMAGE_MAGIC_VAL  ((int)sizeof(IplImage))


#define IPL_IMAGE_FILE_MAGIC_VAL  112

#define CV_IS_IMAGE_HDR(img)     ((img) != NULL && ((const IplImage*)(img))->nSize == sizeof(IplImage))

#define CV_IS_IMAGE(img)     (CV_IS_IMAGE_HDR(img) && ((IplImage*)img)->imageData != NULL)

#define IPL_DEPTH_64F  64


#define CV_IMAGE_ELEM( image, elemtype, row, col )     (((elemtype*)((image)->imageData + (image)->widthStep*(row))[(col)]))




#define CV_8U   0
#define CV_8S   1
#define CV_16S  2
#define CV_32S  3
#define CV_32F  4
#define CV_64F  5
#define CV_USRTYPE1 6
#define CV_USRTYPE2 7

#define CV_8UC1 (CV_8U + 0*8)
#define CV_8UC2 (CV_8U + 1*8)
#define CV_8UC3 (CV_8U + 2*8)
#define CV_8UC4 (CV_8U + 3*8)

#define CV_8SC1 (CV_8S + 0*8)
#define CV_8SC2 (CV_8S + 1*8)
#define CV_8SC3 (CV_8S + 2*8)
#define CV_8SC4 (CV_8S + 3*8)

#define CV_16SC1 (CV_16S + 0*8)
#define CV_16SC2 (CV_16S + 1*8)
#define CV_16SC3 (CV_16S + 2*8)
#define CV_16SC4 (CV_16S + 3*8)

#define CV_32SC1 (CV_32S + 0*8)
#define CV_32SC2 (CV_32S + 1*8)
#define CV_32SC3 (CV_32S + 2*8)
#define CV_32SC4 (CV_32S + 3*8)

#define CV_32FC1 (CV_32F + 0*8)
#define CV_32FC2 (CV_32F + 1*8)
#define CV_32FC3 (CV_32F + 2*8)
#define CV_32FC4 (CV_32F + 3*8)

#define CV_64FC1 (CV_64F + 0*8)
#define CV_64FC2 (CV_64F + 1*8)
#define CV_64FC3 (CV_64F + 2*8)
#define CV_64FC4 (CV_64F + 3*8)

#define CV_AUTO_STEP  0x7fffffff
#define CV_WHOLE_ARR  cvSlice( 0, 0x3fffffff )

#define CV_MAT_CN_MASK          (3 << 3)
#define CV_MAT_CN(flags)        ((((flags) & CV_MAT_CN_MASK) >> 3) + 1)
#define CV_MAT_DEPTH_MASK       7
#define CV_MAT_DEPTH(flags)     ((flags) & CV_MAT_DEPTH_MASK)
#define CV_MAT_TYPE_MASK        31
#define CV_MAT_TYPE(flags)      ((flags) & CV_MAT_TYPE_MASK)
#define CV_MAT_FMT_MASK         511
#define CV_MAT_FMT(flags)       ((flags) & CV_MAT_FMT_MASK)
#define CV_MAT_CONT_FLAG_SHIFT  9
#define CV_MAT_CONT_FLAG        (1 << CV_MAT_CONT_FLAG_SHIFT)
#define CV_IS_MAT_CONT(flags)   ((flags) & CV_MAT_CONT_FLAG)
#define CV_IS_CONT_MAT          CV_IS_MAT_CONT
#define CV_MAT_TEMP_FLAG_SHIFT  10
#define CV_MAT_TEMP_FLAG        (1 << CV_MAT_TEMP_FLAG_SHIFT)
#define CV_IS_TEMP_MAT(flags)   ((flags) & CV_MAT_TEMP_FLAG)

#define CV_MAGIC_MASK       0xFFFF0000
#define CV_MAT_MAGIC_VAL    0x42420000

typedef union {
    uchar* ptr;
    short* s;
    int* i;
    float* fl;
    double* db;
} CvMat_data;

typedef struct CvMat
{
    int type;
    int step;

    int* refcount;
    CvMat_data data;
/*    union
    {
        uchar* ptr;
        short* s;
        int* i;
        float* fl;
        double* db;
    } data; */

    int rows;
    int cols;

} CvMat;


#define CV_IS_MAT_HDR(mat)     ((mat) != NULL && (((const CvMat*)(mat))->type & CV_MAGIC_MASK) == CV_MAT_MAGIC_VAL)

#define CV_IS_MAT(mat)     (CV_IS_MAT_HDR(mat) && ((const CvMat*)(mat))->data.ptr != NULL)

#define CV_IS_MASK_ARR(mat)     (((mat)->type & (CV_MAT_TYPE_MASK & ~CV_8SC1)) == 0)

#define CV_ARE_TYPES_EQ(mat1, mat2)     ((((mat1)->type ^ (mat2)->type) & CV_MAT_TYPE_MASK) == 0)

#define CV_ARE_CNS_EQ(mat1, mat2)     ((((mat1)->type ^ (mat2)->type) & CV_MAT_CN_MASK) == 0)

#define CV_ARE_DEPTHS_EQ(mat1, mat2)     ((((mat1)->type ^ (mat2)->type) & CV_MAT_DEPTH_MASK) == 0)

#define CV_ARE_SIZES_EQ(mat1, mat2)     ((mat1)->height == (mat2)->height && (mat1)->width == (mat2)->width)

#define CV_IS_MAT_CONST(mat)     (((mat)->height|(mat)->width) == 1)

#define CV_ELEM_SIZE(type)     (CV_MAT_CN(type) << ((0xe90 >> CV_MAT_DEPTH(type)*2) & 3))


CvMat cvMat( int rows, int cols, int type, void* data );

#define CV_MAT_ELEM_PTR_FAST( mat, row, col, pix_size )     (assert( (unsigned)(row) < (unsigned)(mat).rows &&              (unsigned)(col) < (unsigned)(mat).cols ),      (mat).data.ptr + (size_t)(mat).step*(row) + (pix_size)*(col))

#define CV_MAT_ELEM_PTR( mat, row, col )     CV_MAT_ELEM_PTR_FAST( mat, row, col, CV_ELEM_SIZE((mat).type) )

#define CV_MAT_ELEM( mat, elemtype, row, col )     (*(elemtype*)CV_MAT_ELEM_PTR_FAST( mat, row, col, sizeof(elemtype)))


double  cvmGet( const CvMat* mat, int i, int j );

void  cvmSet( CvMat* mat, int i, int j, double val );

#define CV_MATND_MAGIC_VAL    0x42430000
#define CV_MAX_DIM 16

#define CV_MAT_LIKE_FLAG_SHIFT  11
#define CV_MAT_LIKE_FLAG        (1 << CV_MAT_LIKE_FLAG_SHIFT)

typedef struct {
    int size;
    int step;
} CvMatND_dim;

typedef struct CvMatND
{
    int type;
    int dims;

    int* refcount;
    CvMat_data data;
/*    union
    {
        uchar* ptr;
        float* fl;
        double* db;
        int* i;
        short* s;
    } data; */
    CvMatND_dim dim[CV_MAX_DIM];
/*    struct
    {
        int size;
        int step;
    }
    dim[CV_MAX_DIM]; */
}
CvMatND;

#define CV_IS_MATND_HDR(mat)     ((mat) != NULL && (((const CvMatND*)(mat))->type & CV_MAGIC_MASK) == CV_MATND_MAGIC_VAL)

#define CV_IS_MATND(mat)     (CV_IS_MATND_HDR(mat) && ((const CvMatND*)(mat))->data.ptr != NULL)




#define CV_SPARSE_MAT_MAGIC_VAL    0x42440000

struct CvSet;

typedef struct CvSparseMat
{
    int type;
    int dims;
    int* refcount;
    struct CvSet* heap;
    void** hashtable;
    int hashsize;
    int total;
    int valoffset;
    int idxoffset;
    int size[CV_MAX_DIM];
}
CvSparseMat;

#define CV_IS_SPARSE_MAT_HDR(mat)     ((mat) != NULL &&     (((const CvSparseMat*)(mat))->type & CV_MAGIC_MASK) == CV_SPARSE_MAT_MAGIC_VAL)

#define CV_IS_SPARSE_MAT(mat)     CV_IS_SPARSE_MAT_HDR(mat)



typedef struct CvSparseNode
{
    unsigned hashval;
    struct CvSparseNode* next;
}
CvSparseNode;

typedef struct CvSparseMatIterator
{
    CvSparseMat* mat;
    CvSparseNode* node;
    int curidx;
}
CvSparseMatIterator;

#define CV_NODE_VAL(mat,node)   ((void*)((uchar*)(node) + (mat)->valoffset))
#define CV_NODE_IDX(mat,node)   ((int*)((uchar*)(node) + (mat)->idxoffset))



typedef int CvHistType;

#define CV_HIST_MAGIC_VAL     0x42450000
#define CV_HIST_UNIFORM_FLAG  (1 << 10)


#define CV_HIST_RANGES_FLAG   (1 << 11)

#define CV_HIST_ARRAY         0
#define CV_HIST_SPARSE        1
#define CV_HIST_TREE          CV_HIST_SPARSE

#define CV_HIST_UNIFORM       1

typedef struct CvHistogram
{
    int     type;
    CvArr*  bins;
    float   thresh[CV_MAX_DIM][2];
    float** thresh2;
    CvMatND mat;
}
CvHistogram;

#define CV_IS_HIST( hist )     ((hist) != NULL  &&      (((CvHistogram*)(hist))->type & CV_MAGIC_MASK) == CV_HIST_MAGIC_VAL &&      (hist)->bins != NULL)

#define CV_IS_UNIFORM_HIST( hist )     (((hist)->type & CV_HIST_UNIFORM_FLAG) != 0)

#define CV_IS_SPARSE_HIST( hist )     CV_IS_SPARSE_MAT((hist)->bins)

#define CV_HIST_HAS_RANGES( hist )     (((hist)->type & CV_HIST_RANGES_FLAG) != 0)




int  cvRound( double val );
int  cvFloor( double val );
int  cvCeil( double val );

typedef struct CvRect
{
    int x;
    int y;
    int width;
    int height;
}
CvRect;

CvRect  cvRect( int x, int y, int width, int height );
IplROI  cvRectToROI( CvRect rect, int coi );
CvRect  cvROIToRect( IplROI roi );

#define CV_TERMCRIT_ITER    1
#define CV_TERMCRIT_NUMB    CV_TERMCRIT_ITER
#define CV_TERMCRIT_EPS     2

typedef struct CvTermCriteria
{
    int    type;
    int    maxIter;
    double epsilon;
}
CvTermCriteria;

CvTermCriteria  cvTermCriteria( int type, int maxIter, double epsilon );

typedef struct CvPoint
{
    int x;
    int y;
}
CvPoint;


CvPoint  cvPoint( int x, int y );

typedef struct CvPoint2D32f
{
    float x;
    float y;
}
CvPoint2D32f;


CvPoint2D32f  cvPoint2D32f( double x, double y );
CvPoint2D32f  cvPointTo32f( CvPoint pt );
CvPoint  cvPointFrom32f( CvPoint2D32f pt );

typedef struct CvPoint3D32f
{
    float x;
    float y;
    float z;
}
CvPoint3D32f;


CvPoint3D32f  cvPoint3D32f( double x, double y, double z );

typedef struct CvPoint2D64d
{
    double x;
    double y;
}
CvPoint2D64d;


typedef struct CvPoint3D64d
{
    double x;
    double y;
    double z;
}
CvPoint3D64d;




typedef struct
{
    int width;
    int height;
}
CvSize;

CvSize  cvSize( int width, int height );

typedef struct CvSize2D32f
{
    float width;
    float height;
}
CvSize2D32f;


CvSize2D32f  cvSize2D32f( double width, double height );

typedef struct CvBox2D
{
    CvPoint2D32f center;
    CvSize2D32f  size;
    float angle;
}
CvBox2D;



typedef struct CvSlice
{
    int  startIndex, endIndex;
} CvSlice;

CvSlice  cvSlice( int start, int end );

#define CV_WHOLE_SEQ  cvSlice(0, 0x3fffffff)




typedef struct CvScalar
{
    double val[4];
}
CvScalar;


CvScalar  cvScalar( double a, double b ,
                               double c , double d );

CvScalar  cvRealScalar( double a );
CvScalar  cvScalarAll( double a );

struct CvSeq;

typedef struct CvConnectedComp
{
    double area;
    double value;
    CvRect rect;
    struct CvSeq* contour;
}
CvConnectedComp;




#define CV_PI   3.1415926535897932384626433832795

#define CV_SWAP(a,b,t) ((t) = (a), (a) = (b), (b) = (t))


#define  CV_IMIN(a, b)  ((a) ^ (((a)^(b)) & (((a) < (b)) - 1)))

#define  CV_IMAX(a, b)  ((a) ^ (((a)^(b)) & (((a) > (b)) - 1)))


#define  CV_IABS(a)     (((a) ^ ((a) < 0 ? -1 : 0)) - ((a) < 0 ? -1 : 0))
#define  CV_SIGN(a)     (((a) < 0 ? -1 : 0) | ((a) > 0))


#define  CV_INIT_3X3_DELTAS( deltas, step, nch )     ((deltas)[0] =  (nch),  (deltas)[1] = -(step) + (nch),      (deltas)[2] = -(step), (deltas)[3] = -(step) - (nch),      (deltas)[4] = -(nch),  (deltas)[5] =  (step) - (nch),      (deltas)[6] =  (step), (deltas)[7] =  (step) + (nch))


#define CV_SQRT_MAGIC  0xbe6f0000

float  cvInvSqrt( float arg );
float  cvSqrt( float arg );

typedef enum CvDisType
{
    CV_DIST_USER         = -1,
    CV_DIST_L1           = 1,
    CV_DIST_L2           = 2,
    CV_DIST_C            = 3,
    CV_DIST_L12          = 4,
    CV_DIST_FAIR         = 5,
    CV_DIST_WELSCH       = 6,
    CV_DIST_HUBER        = 7
}
CvDisType;



typedef enum CvFilter
{
    CV_GAUSSIAN_5x5 = 7
}
CvFilter;







typedef struct CvMemBlock
{
    struct CvMemBlock*  prev;
    struct CvMemBlock*  next;
}
CvMemBlock;

#define CV_STORAGE_MAGIC_VAL    0x42890000

typedef struct CvMemStorage
{
    int     signature;
    CvMemBlock* bottom;
    CvMemBlock* top;
    struct  CvMemStorage* parent;
    int     block_size;
    int     free_space;
}
CvMemStorage;

#define CV_IS_STORAGE(storage)     ((storage) != NULL &&     (((CvMemStorage*)(storage))->signature & CV_MAGIC_MASK) == CV_STORAGE_MAGIC_VAL)


typedef struct CvMemStoragePos
{
    CvMemBlock* top;
    int  free_space;
}
CvMemStoragePos;




typedef struct CvSeqBlock
{
    struct CvSeqBlock*  prev;
    struct CvSeqBlock*  next;
    int    start_index;
    int    count;
    char*  data;}
CvSeqBlock;

typedef struct CvSeq
{
/*  CV_SEQUENCE_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
}
CvSeq;

typedef struct CvSetElem
{
/*  CV_SET_ELEM_FIELDS(CvSetElem) */
    int  flags;
    struct CvSetElem* next_free;
}
CvSetElem;

typedef struct CvSet
{
/*  CV_SET_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvSetElem* free_elems;
}
CvSet;


#define CV_SET_ELEM_IDX_MASK   ((1 << 24) - 1)
#define CV_SET_ELEM_FREE_FLAG  (1 << (sizeof(int)*8-1))


#define CV_IS_SET_ELEM( ptr )  (((CvSetElem*)(ptr))->flags >= 0)

typedef struct CvGraphEdge
{
/*  CV_GRAPH_EDGE_FIELDS() */
    int flags;
    float weight;
    struct CvGraphEdge* next[2];
    struct CvGraphVtx* vtx[2];
}
CvGraphEdge;

typedef struct CvGraphVtx
{
/*  CV_GRAPH_VERTEX_FIELDS() */
    int flags;
    struct CvGraphEdge* first;
}
CvGraphVtx;

typedef struct CvGraphVtx2D
{
/*  CV_GRAPH_VERTEX_FIELDS() */
    int flags;
    struct CvGraphEdge* first;
    CvPoint2D32f* ptr;
}
CvGraphVtx2D;

typedef struct CvGraph
{
/*  CV_GRAPH_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvSetElem* free_elems;
    CvSet* edges;
}
CvGraph;



typedef struct CvChain
{
/*  CV_SEQUENCE_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvPoint  origin;
}
CvChain;

typedef struct CvContour
{
/*  CV_CONTOUR_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvRect rect;
    int color;
    int reserved[3];
}
CvContour;

typedef CvContour CvPoint2DSeq;



#define CV_SEQ_MAGIC_VAL             0x42990000
#define CV_IS_SEQ(seq)     ((seq) != NULL && (((CvSeq*)(seq))->flags & CV_MAGIC_MASK) == CV_SEQ_MAGIC_VAL)

#define CV_SET_MAGIC_VAL             0x42980000
#define CV_IS_SET(set)     ((set) != NULL && (((CvSeq*)(set))->flags & CV_MAGIC_MASK) == CV_SET_MAGIC_VAL)

#define CV_SEQ_ELTYPE_BITS           5
#define CV_SEQ_ELTYPE_MASK           ((1 << CV_SEQ_ELTYPE_BITS) - 1)

#define CV_SEQ_ELTYPE_POINT          CV_32SC2
#define CV_SEQ_ELTYPE_CODE           CV_8UC1
#define CV_SEQ_ELTYPE_GENERIC        0
#define CV_SEQ_ELTYPE_PTR            CV_USRTYPE1
#define CV_SEQ_ELTYPE_PPOINT         CV_SEQ_ELTYPE_PTR
#define CV_SEQ_ELTYPE_INDEX          CV_32SC1
#define CV_SEQ_ELTYPE_GRAPH_EDGE     0
#define CV_SEQ_ELTYPE_GRAPH_VERTEX   0
#define CV_SEQ_ELTYPE_TRIAN_ATR      0
#define CV_SEQ_ELTYPE_CONNECTED_COMP 0
#define CV_SEQ_ELTYPE_POINT3D        CV_32FC3

#define CV_SEQ_KIND_BITS        5
#define CV_SEQ_KIND_MASK        (((1 << CV_SEQ_KIND_BITS) - 1)<<CV_SEQ_ELTYPE_BITS)


#define CV_SEQ_KIND_GENERIC     (0 << CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_CURVE       (1 << CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_BIN_TREE    (2 << CV_SEQ_ELTYPE_BITS)


#define CV_SEQ_KIND_GRAPH       (3 << CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_SUBDIV2D    (4 << CV_SEQ_ELTYPE_BITS)

#define CV_SEQ_FLAG_SHIFT       (CV_SEQ_KIND_BITS + CV_SEQ_ELTYPE_BITS)


#define CV_SEQ_FLAG_CLOSED     (1 << CV_SEQ_FLAG_SHIFT)
#define CV_SEQ_FLAG_SIMPLE     (2 << CV_SEQ_FLAG_SHIFT)
#define CV_SEQ_FLAG_CONVEX     (4 << CV_SEQ_FLAG_SHIFT)
#define CV_SEQ_FLAG_HOLE       (8 << CV_SEQ_FLAG_SHIFT)


#define CV_GRAPH_FLAG_ORIENTED (1 << CV_SEQ_FLAG_SHIFT)

#define CV_GRAPH               CV_SEQ_KIND_GRAPH
#define CV_ORIENTED_GRAPH      (CV_SEQ_KIND_GRAPH|CV_GRAPH_FLAG_ORIENTED)


#define CV_SEQ_POINT_SET       (CV_SEQ_KIND_GENERIC| CV_SEQ_ELTYPE_POINT)
#define CV_SEQ_POINT3D_SET     (CV_SEQ_KIND_GENERIC| CV_SEQ_ELTYPE_POINT3D)
#define CV_SEQ_POLYLINE        (CV_SEQ_KIND_CURVE  | CV_SEQ_ELTYPE_POINT)
#define CV_SEQ_POLYGON         (CV_SEQ_FLAG_CLOSED | CV_SEQ_POLYLINE )
#define CV_SEQ_CONTOUR         CV_SEQ_POLYGON
#define CV_SEQ_SIMPLE_POLYGON  (CV_SEQ_FLAG_SIMPLE | CV_SEQ_POLYGON  )


#define CV_SEQ_CHAIN           (CV_SEQ_KIND_CURVE  | CV_SEQ_ELTYPE_CODE)
#define CV_SEQ_CHAIN_CONTOUR   (CV_SEQ_FLAG_CLOSED | CV_SEQ_CHAIN)


#define CV_SEQ_POLYGON_TREE    (CV_SEQ_KIND_BIN_TREE  | CV_SEQ_ELTYPE_TRIAN_ATR)


#define CV_SEQ_CONNECTED_COMP  (CV_SEQ_KIND_GENERIC  | CV_SEQ_ELTYPE_CONNECTED_COMP)


#define CV_SEQ_INDEX           (CV_SEQ_KIND_GENERIC  | CV_SEQ_ELTYPE_INDEX)

#define CV_SEQ_ELTYPE( seq )   ((seq)->flags & CV_SEQ_ELTYPE_MASK)
#define CV_SEQ_KIND( seq )     ((seq)->flags & CV_SEQ_KIND_MASK )


#define CV_IS_SEQ_INDEX( seq )      ((CV_SEQ_ELTYPE(seq) == CV_SEQ_ELTYPE_INDEX) &&                                      (CV_SEQ_KIND(seq) == CV_SEQ_KIND_GENERIC))

#define CV_IS_SEQ_CURVE( seq )      (CV_SEQ_KIND(seq) == CV_SEQ_KIND_CURVE)
#define CV_IS_SEQ_CLOSED( seq )     (((seq)->flags & CV_SEQ_FLAG_CLOSED) != 0)
#define CV_IS_SEQ_CONVEX( seq )     (((seq)->flags & CV_SEQ_FLAG_CONVEX) != 0)
#define CV_IS_SEQ_HOLE( seq )       (((seq)->flags & CV_SEQ_FLAG_HOLE) != 0)
#define CV_IS_SEQ_SIMPLE( seq )     ((((seq)->flags & CV_SEQ_FLAG_SIMPLE) != 0) ||                                     CV_IS_SEQ_CONVEX(seq))


#define CV_IS_SEQ_POINT_SET( seq )     ((CV_SEQ_ELTYPE(seq) == CV_32SC2 || CV_SEQ_ELTYPE(seq) == CV_32FC2))

#define CV_IS_SEQ_POINT_SUBSET( seq )     (CV_IS_SEQ_INDEX( seq ) || CV_SEQ_ELTYPE(seq) == CV_SEQ_ELTYPE_PPOINT)

#define CV_IS_SEQ_POLYLINE( seq )     (CV_SEQ_KIND(seq) == CV_SEQ_KIND_CURVE && CV_IS_SEQ_POINT_SET(seq))

#define CV_IS_SEQ_POLYGON( seq )     (CV_IS_SEQ_POLYLINE(seq) && CV_IS_SEQ_CLOSED(seq))

#define CV_IS_SEQ_CHAIN( seq )     (CV_SEQ_KIND(seq) == CV_SEQ_KIND_CURVE && (seq)->elem_size == 1)

#define CV_IS_SEQ_CONTOUR( seq )     (CV_IS_SEQ_CLOSED(seq) && (CV_IS_SEQ_POLYLINE(seq) || CV_IS_SEQ_CHAIN(seq)))

#define CV_IS_SEQ_CHAIN_CONTOUR( seq )     (CV_IS_SEQ_CHAIN( seq ) && CV_IS_SEQ_CLOSED( seq ))

#define CV_IS_SEQ_POLYGON_TREE( seq )     (CV_SEQ_ELTYPE (seq) ==  CV_SEQ_ELTYPE_TRIAN_ATR &&     CV_SEQ_KIND( seq ) ==  CV_SEQ_KIND_BIN_TREE )

#define CV_IS_GRAPH( seq )     (CV_IS_SET(seq) && CV_SEQ_KIND((CvSet*)(seq)) == CV_SEQ_KIND_GRAPH)

#define CV_IS_GRAPH_ORIENTED( seq )     (((seq)->flags & CV_GRAPH_FLAG_ORIENTED) != 0)

#define CV_IS_SUBDIV2D( seq )     (CV_IS_SET(seq) && CV_SEQ_KIND((CvSet*)(seq)) == CV_SEQ_KIND_SUBDIV2D)

typedef struct CvSeqWriter
{
/*  CV_SEQ_WRITER_FIELDS() */
    int          header_size;
    CvSeq*       seq;
    CvSeqBlock*  block;
    char*        ptr;
    char*        block_min;
    char*        block_max;
    int  reserved[4];
}
CvSeqWriter;

typedef struct CvSeqReader
{
/*  CV_SEQ_READER_FIELDS() */
    int          header_size;
    CvSeq*       seq;
    CvSeqBlock*  block;
    char*        ptr;
    char*        block_min;
    char*        block_max;
    int          delta_index;
    char*        prev_elem;
    int  reserved[4];
}
CvSeqReader;

#define  CV_GET_SEQ_ELEM( elem_type, seq, index )  (   assert(sizeof((seq)->first[0]) == sizeof(CvSeqBlock) &&     (seq)->elem_size == sizeof(elem_type)),     (elem_type*)((seq)->first && (unsigned)index <     (unsigned)((seq)->first->count) ?     (seq)->first->data + (index) * sizeof(elem_type) :     cvGetSeqElem( (CvSeq*)(seq), (index), NULL )))



#define CV_WRITE_SEQ_ELEM_VAR( elem_ptr, writer ) {     if( (writer).ptr >= (writer).block_max )     {         cvCreateSeqBlock( &writer);     }     memcpy((writer).ptr, elem_ptr, (writer).seq->elem_size);     (writer).ptr += (writer).seq->elem_size; }

#define CV_WRITE_SEQ_ELEM( elem, writer ) {     assert( (writer).seq->elem_size == sizeof(elem));     if( (writer).ptr >= (writer).block_max )     {         cvCreateSeqBlock( &writer);     }     assert( (writer).ptr <= (writer).block_max - sizeof(elem));     memcpy((writer).ptr, &elem, sizeof(elem));     (writer).ptr += sizeof(elem); }



#define CV_NEXT_SEQ_ELEM( elem_size, reader ) {     if( ((reader).ptr += (elem_size)) >= (reader).block_max )     {         cvChangeSeqBlock( &(reader), 1 );     } }



#define CV_PREV_SEQ_ELEM( elem_size, reader ) {     if( ((reader).ptr -= (elem_size)) < (reader).block_min )     {         cvChangeSeqBlock( &(reader), -1 );     } }


#define CV_READ_SEQ_ELEM( elem, reader ) {     assert( (reader).seq->elem_size == sizeof(elem));     memcpy( &(elem), (reader).ptr, sizeof((elem)));     CV_NEXT_SEQ_ELEM( sizeof(elem), reader ) }


#define CV_REV_READ_SEQ_ELEM( elem, reader ) {     assert( (reader).seq->elem_size == sizeof(elem));     memcpy(&(elem), (reader).ptr, sizeof((elem)));     CV_PREV_SEQ_ELEM( sizeof(elem), reader ) }


#define CV_READ_CHAIN_POINT( _pt, reader ) {     (_pt) = (reader).pt;     if( (reader).ptr )     {         CV_READ_SEQ_ELEM( (reader).code, (*((CvSeqReader*)&(reader))));         assert( ((reader).code & ~7) == 0 );         (reader).pt.x += (reader).deltas[(reader).code][0];         (reader).pt.y += (reader).deltas[(reader).code][1];     } }

#define CV_CURRENT_POINT( reader )  (*((CvPoint*)((reader).ptr)))
#define CV_PREV_POINT( reader )     (*((CvPoint*)((reader).prev_elem)))

#define CV_READ_EDGE( pt1, pt2, reader ) {     assert( sizeof(pt1) == sizeof(CvPoint) &&             sizeof(pt2) == sizeof(CvPoint) &&             reader.seq->elem_size == sizeof(CvPoint));     (pt1) = CV_PREV_POINT( reader );     (pt2) = CV_CURRENT_POINT( reader );     (reader).prev_elem = (reader).ptr;     CV_NEXT_SEQ_ELEM( sizeof(CvPoint), (reader)); }




#define  CV_NEXT_GRAPH_EDGE( edge, vertex )      (assert((edge)->vtx[0] == (vertex) || (edge)->vtx[1] == (vertex)),       (edge)->next[(edge)->vtx[1] == (vertex)])




typedef struct
{
    const char* file;
    int         line;
}
CvStackRecord;






typedef float*   CvVect32f;
typedef float*   CvMatr32f;
typedef double*  CvVect64d;
typedef double*  CvMatr64d;

typedef struct CvMatrix3
{
    float m[3][3];
}
CvMatrix3;

typedef enum CvStatus
{
    CV_INPLACE_NOT_SUPPORTED_ERR= -112,
    CV_UNMATCHED_ROI_ERR        = -111,
    CV_NOTFOUND_ERR             = -110,
    CV_BADCONVERGENCE_ERR       = -109,

    CV_BADDEPTH_ERR             = -107,
    CV_BADROI_ERR               = -106,
    CV_BADHEADER_ERR            = -105,
    CV_UNMATCHED_FORMATS_ERR    = -104,
    CV_UNSUPPORTED_COI_ERR      = -103,
    CV_UNSUPPORTED_CHANNELS_ERR = -102,
    CV_UNSUPPORTED_DEPTH_ERR    = -101,
    CV_UNSUPPORTED_FORMAT_ERR   = -100,


    CV_BADARG_ERR      = -49,
    CV_NOTDEFINED_ERR  = -48,

    CV_BADCHANNELS_ERR = -47,
    CV_BADRANGE_ERR    = -44,
    CV_BADSTEP_ERR     = -29,

    CV_BADFLAG_ERR     =  -12,
    CV_DIV_BY_ZERO_ERR =  -11,
    CV_BADCOEF_ERR     =  -10,

    CV_BADFACTOR_ERR   =  -7,
    CV_BADPOINT_ERR    =  -6,
    CV_BADSCALE_ERR    =  -4,
    CV_OUTOFMEM_ERR    =  -3,
    CV_NULLPTR_ERR     =  -2,
    CV_BADSIZE_ERR     =  -1,
    CV_NO_ERR          =   0,
    CV_OK              =   CV_NO_ERR
}
CvStatus;

/******************************************************************/
/*cverror.h */

typedef int CVStatus;

#define CV_StsOk                    0
#define CV_StsBackTrace            -1
#define CV_StsError                -2
#define CV_StsInternal             -3
#define CV_StsNoMem                -4
#define CV_StsBadArg               -5
#define CV_StsBadFunc              -6
#define CV_StsNoConv               -7
#define CV_StsAutoTrace            -8

#define CV_HeaderIsNull            -9
#define CV_BadImageSize            -10
#define CV_BadOffset               -11
#define CV_BadDataPtr              -12
#define CV_BadStep                 -13
#define CV_BadModelOrChSeq         -14
#define CV_BadNumChannels          -15
#define CV_BadNumChannel1U         -16
#define CV_BadDepth                -17
#define CV_BadAlphaChannel         -18
#define CV_BadOrder                -19
#define CV_BadOrigin               -20
#define CV_BadAlign                -21
#define CV_BadCallBack             -22
#define CV_BadTileSize             -23
#define CV_BadCOI                  -24
#define CV_BadROISize              -25

#define CV_MaskIsTiled             -26

#define CV_StsNullPtr                -27
#define CV_StsVecLengthErr           -28
#define CV_StsFilterStructContentErr -29
#define CV_StsKernelStructContentErr -30
#define CV_StsFilterOffsetErr        -31


#define CV_StsBadSize                -201
#define CV_StsDivByZero              -202
#define CV_StsInplaceNotSupported    -203
#define CV_StsObjectNotFound         -204
#define CV_StsUnmatchedFormats       -205
#define CV_StsBadFlag                -206
#define CV_StsBadPoint               -207
#define CV_StsBadMask                -208
#define CV_StsUnmatchedSizes         -209
#define CV_StsUnsupportedFormat      -210
#define CV_StsOutOfRange             -211

typedef int ( *CVErrorCallBack) (CVStatus status, const char *func,
                                         const char *context, const char *file,int line);

#define CV_ErrModeLeaf     0
#define CV_ErrModeParent   1
#define CV_ErrModeSilent   2

/******************************************************************/
/* cv.h */

void*  cvAlloc( size_t size );


void   cvFree( void** ptr );


IplImage*  cvCreateImageHeader( CvSize size, int depth, int channels );


IplImage* cvInitImageHeader( IplImage* image, CvSize size, int depth,
                                       int channels, int origin ,
                                       int align );


IplImage*  cvCreateImage( CvSize size, int depth, int channels );


void  cvReleaseImageHeader( IplImage** image );


void  cvReleaseImage( IplImage** image );


IplImage* cvCloneImage( const IplImage* image );


void  cvSetImageCOI( IplImage* image, int coi );


int  cvGetImageCOI( IplImage* image );


void  cvSetImageROI( IplImage* image, CvRect rect );


void  cvResetImageROI( IplImage* image );


CvRect cvGetImageROI( const IplImage* image );


CvMat*  cvCreateMatHeader( int rows, int cols, int type );

#define CV_AUTOSTEP  0x7fffffff


CvMat* cvInitMatHeader( CvMat* mat, int rows, int cols,
                                  int type, void* data ,
                                  int step );


CvMat*  cvCreateMat( int rows, int cols, int type );


void  cvReleaseMat( CvMat** mat );


void  cvDecRefData( CvArr* arr );
int  cvIncRefData( CvArr* arr );

CvMat* cvCloneMat( const CvMat* mat );



CvMat* cvGetSubRect( const CvArr* arr, CvMat* submat, CvRect rect );
#define cvGetSubArr cvGetSubRect


CvMat* cvGetRows( const CvArr* arr, CvMat* submat,
                            int start_row, int end_row );

void  cvGetRow( const CvArr* arr, CvMat* submat, int row );

CvMat* cvGetCols( const CvArr* arr, CvMat* submat,
                               int start_col, int end_col );

void  cvGetCol( const CvArr* arr, CvMat* submat, int col );

CvMat* cvGetDiag( const CvArr* arr, CvMat* submat,
                            int diag );


void cvScalarToRawData( const CvScalar* scalar, void* data, int type,
                                  int extend_to_12  );

void cvRawDataToScalar( const void* data, int type, CvScalar* scalar );


CvMatND*  cvCreateMatNDHeader( int dims, const int* sizes, int type );


CvMatND*  cvCreateMatND( int dims, const int* sizes, int type );


CvMatND*  cvInitMatNDHeader( CvMatND* mat, int dims, const int* sizes,
                                        int type, void* data  );


void  cvReleaseMatND( CvMatND** mat );

CvMatND* cvCloneMatND( const CvMatND* mat );


CvSparseMat*  cvCreateSparseMat( int dims, const int* sizes, int type );


void  cvReleaseSparseMat( CvSparseMat** mat );


CvSparseMat* cvCloneSparseMat( const CvSparseMat* mat );


CvSparseNode* cvInitSparseMatIterator( const CvSparseMat* mat,
                                                  CvSparseMatIterator* matIterator );

CvSparseNode* cvGetNextSparseNode( CvSparseMatIterator* matIterator );

int cvGetElemType( const CvArr* arr );


int cvGetDims( const CvArr* arr, int* sizes  );



int cvGetDimSize( const CvArr* arr, int index );



uchar* cvPtr1D( const CvArr* arr, int idx1, int* type );
uchar* cvPtr2D( const CvArr* arr, int idx1, int idx2, int* type  );
uchar* cvPtr3D( const CvArr* arr, int idx1, int idx2, int idx3,
                          int* type );


uchar* cvPtrND( const CvArr* arr, int* idx, int* type  );


CvScalar cvGet1D( const CvArr* arr, int idx1 );
CvScalar cvGet2D( const CvArr* arr, int idx1, int idx2 );
CvScalar cvGet3D( const CvArr* arr, int idx1, int idx2, int idx3 );
CvScalar cvGetND( const CvArr* arr, int* idx );


double cvGetReal1D( const CvArr* arr, int idx1 );
double cvGetReal2D( const CvArr* arr, int idx1, int idx2 );
double cvGetReal3D( const CvArr* arr, int idx1, int idx2, int idx3 );
double cvGetRealND( const CvArr* arr, int* idx );


void cvSet1D( CvArr* arr, int idx1, CvScalar value );
void cvSet2D( CvArr* arr, int idx1, int idx2, CvScalar value );
void cvSet3D( CvArr* arr, int idx1, int idx2, int idx3, CvScalar value );
void cvSetND( CvArr* arr, int* idx, CvScalar value );


void cvSetReal1D( CvArr* arr, int idx1, double value );
void cvSetReal2D( CvArr* arr, int idx1, int idx2, double value );
void cvSetReal3D( CvArr* arr, int idx1,
                            int idx2, int idx3, double value );
void cvSetRealND( CvArr* arr, int* idx, double value );


void cvClearND( CvArr* arr, int* idx );


CvMat* cvGetMat( const CvArr* src, CvMat* header,
                           int* coi ,
                           int allowND );


IplImage* cvGetImage( const CvArr* arr, IplImage* img );



CvArr* cvReshapeMatND( const CvArr* arr,
                                 int sizeof_header, CvArr* header,
                                 int new_cn, int new_dims, int* new_sizes );

#define cvReshapeND( arr, header, new_cn, new_dims, new_sizes )  cvReshapeMatND( (arr), sizeof(*(header)), (header), (new_cn), (new_dims), (new_sizes))

CvMat* cvReshape( const CvArr* arr, CvMat* header,
                            int new_cn, int new_rows  );


void cvRepeat( const CvArr* src, CvArr* dst );


void  cvCreateData( CvArr* arr );


void  cvReleaseData( CvArr* arr );


void  cvSetData( CvArr* arr, void* data, int step );


void cvGetRawData( const CvArr* arr, uchar** data,
                             int* step ,
                             CvSize* roi_size );


CvSize cvGetSize( const CvArr* arr );


void  cvCopy( const CvArr* src, CvArr* dst,
                         const CvArr* mask  );


void  cvSet( CvArr* arr, CvScalar scalar,
                        const CvArr* mask  );


void  cvSetZero( CvArr* mat );
#define cvZero  cvSetZero



void  cvCvtPixToPlane( const void *src, void *dst0, void *dst1,
                                  void *dst2, void *dst3 );


void  cvCvtPlaneToPix( const void *src0, const void *src1,
                                  const void *src2, const void *src3,
                                  void *dst );


void  cvConvertScale( const CvArr *src, CvArr *dst,
                                 double scale ,
                                 double shift  );
#define cvCvtScale cvConvertScale
#define cvScale  cvConvertScale
#define cvConvert( src, dst )  cvConvertScale( (src), (dst), 1, 0 )



void  cvConvertScaleAbs( const void *src, void *dst,
                                    double scale ,
                                    double shift  );
#define cvCvtScaleAbs  cvConvertScaleAbs



CvRect  cvMaxRect( const CvRect* rect1, const CvRect* rect2 );


void cvBoxPoints( CvBox2D box, CvPoint2D32f pt[4] );





void  cvAdd( const CvArr* srcA, const CvArr* srcB, CvArr* dst,
                        const CvArr* mask );


void  cvAddS( const CvArr* src, CvScalar value, CvArr* dst,
                         const CvArr* mask );


void  cvSub( const CvArr* srcA, const CvArr* srcB, CvArr* dst,
                        const CvArr* mask );


void  cvSubS( const CvArr* src, CvScalar value, CvArr* dst,
                         const CvArr* mask );

void  cvSubRS( const CvArr* src, CvScalar value, CvArr* dst,
                          const CvArr* mask );


void  cvMul( const CvArr* srcA, const CvArr* srcB,
                        CvArr* dst, double scale  );


void  cvDiv( const CvArr* srcA, const CvArr* srcB,
                        CvArr* dst, double scale );


void  cvScaleAdd( const CvArr* srcA, CvScalar scale,
                             const CvArr* srcB, CvArr* dst );


void  cvAddWeighted( const CvArr* srcA, double alpha,
                                const CvArr* srcB, double beta,
                                double gamma, CvArr* dst );


double  cvDotProduct( const CvArr* srcA, const CvArr* srcB );



void cvAnd( const CvArr* src1, const CvArr* src2,
                      CvArr* dst, const CvArr* mask );


void cvAndS( const CvArr* src, CvScalar value,
                       CvArr* dst, const CvArr* mask );


void cvOr( const CvArr* src1, const CvArr* src2,
                     CvArr* dst, const CvArr* mask );


void cvOrS( const CvArr* src, CvScalar value,
                      CvArr* dst, const CvArr* mask );


void cvXor( const CvArr* src1, const CvArr* src2,
                      CvArr* dst, const CvArr* mask );


void cvXorS( const CvArr* src, CvScalar value,
                       CvArr* dst, const CvArr* mask );


void cvNot( const CvArr* src, CvArr* dst );


void cvInRange( const CvArr* src, const CvArr* lower,
                          const CvArr* upper, CvArr* dst );


void cvInRangeS( const CvArr* src, CvScalar lower,
                           CvScalar upper, CvArr* dst );

#define CV_CMP_EQ   0
#define CV_CMP_GT   1
#define CV_CMP_GE   2
#define CV_CMP_LT   3
#define CV_CMP_LE   4
#define CV_CMP_NE   5




void cvCmp( const CvArr* src1, const CvArr* src2, CvArr* dst, int cmpOp );


void cvCmpS( const CvArr* src1, double scalar, CvArr* dst, int cmpOp );


void cvMin( const CvArr* src1, const CvArr* src2, CvArr* dst );


void cvMax( const CvArr* src1, const CvArr* src2, CvArr* dst );


void cvMinS( const CvArr* src, double scalar, CvArr* dst );


void cvMaxS( const CvArr* src, double scalar, CvArr* dst );





void  cvCartToPolar( const CvArr* x, const CvArr* y,
                                CvArr* magnitude, CvArr* angle ,
                                int angle_in_degrees );


void  cvPolarToCart( const CvArr* magnitude, const CvArr* angle,
                                CvArr* x, CvArr* y,
                                int angle_in_degrees );


void  cvPow( const CvArr* src, CvArr* dst, double power );


void  cvExp( const CvArr* src, CvArr* dst );


void  cvLog( const CvArr* src, CvArr* dst );


#define  CV_CHECK_RANGE    1
#define  CV_CHECK_QUIET    2
int  cvCheckArr( const CvArr* arr, int flags ,
                            double minVal , double maxVal );
#define cvCheckArray cvCheckArr


typedef struct CvRandState
{
    uint64    state;
    int       disttype;
    CvScalar  param[2];
}
CvRandState;


#define CV_RAND_UNI      0
#define CV_RAND_NORMAL   1
void  cvRandInit( CvRandState* state, double param1,
                             double param2, int seed,
                             int disttype );


void  cvRandSetRange( CvRandState* state, double param1, double param2,
                                 int index );


void  cvRand( CvRandState* state, CvArr* arr );


unsigned  cvRandNext( CvRandState* state );

void  cvCrossProduct( const CvArr* srcA, const CvArr* srcB, CvArr* dst );


void  cvMatMulAdd( const CvArr* srcA, const CvArr* srcB,
                              const CvArr* srcC, CvArr* dst );
#define cvMatMul( srcA, srcB, dst )  cvMatMulAdd( (srcA), (srcB), NULL, (dst))

#define CV_GEMM_A_T 1
#define CV_GEMM_B_T 2
#define CV_GEMM_C_T 4

void  cvGEMM( const CvArr* srcA, const CvArr* srcB, double alpha,
                         const CvArr* srcC, double beta, CvArr* dst,
                         int tABC );
#define cvMatMulAddEx cvGEMM


void  cvMatMulAddS( const CvArr* src, CvArr* dst,
                               const CvMat* transform,
                               const CvMat* shiftvec );
#define cvTransform cvMatMulAddS


void cvMulTransposed( const CvArr* srcarr,
                                CvArr* dstarr, int order );


void  cvTranspose( const CvArr* src, CvArr* dst );
#define cvT cvTranspose



void  cvFlip( const CvArr* src, CvArr* dst ,
                         int flip_mode );
#define cvMirror cvFlip


#define CV_SVD_MODIFY_A   1
#define CV_SVD_U_T        2
#define CV_SVD_V_T        4


void   cvSVD( CvArr* A, CvArr* W ,
                         CvArr* U ,
                         CvArr* V ,
                         int flags );


void   cvSVBkSb( const CvArr* warr, const CvArr* uarr,
                            const CvArr* varr, const CvArr* barr,
                            CvArr* xarr, int flags );

#define CV_LU  0
#define CV_SVD 1

double  cvInvert( const CvArr* src, CvArr* dst,
                             int method );
#define cvInv cvInvert


int  cvSolve( const CvArr* A, const CvArr* b, CvArr* x,
                         int method );


double cvDet( const CvArr* mat );


CvScalar cvTrace( const CvArr* mat );


void  cvEigenVV( CvArr* src, CvArr* evects, CvArr* evals, double eps );


void  cvSetIdentity( CvArr* mat, CvScalar value );


void  cvPerspectiveTransform( const CvArr* src, CvArr* dst, const CvArr* mat );


void  cvCalcCovarMatrix( const CvArr** vects, CvArr* covarMatrix, CvArr* avg );


double  cvMahalanobis( const CvArr* srcA, const CvArr* srcB, CvArr* mat );
#define cvMahalonobis  cvMahalanobis




CvScalar  cvSum( const CvArr* arr );



int  cvCountNonZero( const CvArr* arr );



CvScalar  cvAvg( const CvArr* arr, const CvArr* mask  );


void  cvAvgSdv( const CvArr* arr, CvScalar* mean, CvScalar* std_dev,
                           const CvArr* mask  );


void  cvMinMaxLoc( const CvArr* arr, double* min_val, double* max_val,
                              CvPoint* min_loc ,
                              CvPoint* max_loc ,
                              const CvArr* mask  );


#define CV_C            1
#define CV_L1           2
#define CV_L2           4
#define CV_NORM_MASK    7
#define CV_RELATIVE     8
#define CV_DIFF         16

#define CV_DIFF_C       (CV_DIFF | CV_C)
#define CV_DIFF_L1      (CV_DIFF | CV_L1)
#define CV_DIFF_L2      (CV_DIFF | CV_L2)
#define CV_RELATIVE_C   (CV_RELATIVE | CV_C)
#define CV_RELATIVE_L1  (CV_RELATIVE | CV_L1)
#define CV_RELATIVE_L2  (CV_RELATIVE | CV_L2)


double  cvNorm( const CvArr* arrA, const CvArr* arrB ,
                           int normType,
                           const CvArr* mask  );



#define CV_DXT_FORWARD  0
#define CV_DXT_INVERSE  1
#define CV_DXT_SCALE    2
#define CV_DXT_INV_SCALE (CV_DXT_INVERSE + CV_DXT_SCALE)
#define CV_DXT_INVERSE_SCALE CV_DXT_INV_SCALE


void  cvDFT( const CvArr* src, CvArr* dst, int flags );
#define cvFFT cvDCT


void  cvMulCcs( const CvArr* srcA, const CvArr* srcB, CvArr* dst );


void  cvDCT( const CvArr* src, CvArr* dst, int flags );




CvMemStorage*  cvCreateMemStorage( int block_size );



CvMemStorage*  cvCreateChildMemStorage( CvMemStorage* parent );



void  cvReleaseMemStorage( CvMemStorage** storage );



void  cvClearMemStorage( CvMemStorage* storage );


void  cvSaveMemStoragePos( const CvMemStorage* storage, CvMemStoragePos* pos );


void  cvRestoreMemStoragePos( CvMemStorage* storage, CvMemStoragePos* pos );


void* cvMemStorageAlloc( CvMemStorage* storage, int size );


CvSeq*  cvCreateSeq( int seq_flags, int header_size,
                             int elem_size, CvMemStorage* storage );


void  cvSetSeqBlockSize( CvSeq* seq, int delta_elements );



char*  cvSeqPush( CvSeq* seq, void* element );



char*  cvSeqPushFront( CvSeq* seq, void* element );



void  cvSeqPop( CvSeq* seq, void* element );



void  cvSeqPopFront( CvSeq* seq, void* element );


#define CV_FRONT 1
#define CV_BACK 0

void  cvSeqPushMulti( CvSeq* seq, void* elements,
                                 int count, int in_front  );


void  cvSeqPopMulti( CvSeq* seq, void* elements,
                                int count, int in_front  );


char*  cvSeqInsert( CvSeq* seq, int before_index,
                               void* element );


void  cvSeqRemove( CvSeq* seq, int index );



void  cvClearSeq( CvSeq* seq );



char*  cvGetSeqElem( CvSeq* seq, int index, CvSeqBlock** block  );



int  cvSeqElemIdx( const CvSeq* seq, const void* element,
                             CvSeqBlock** block  );


void  cvStartAppendToSeq( CvSeq* seq, CvSeqWriter* writer );



void  cvStartWriteSeq( int seq_flags, int header_size,
                                  int elem_size, CvMemStorage* storage,
                                  CvSeqWriter* writer );


CvSeq*  cvEndWriteSeq( CvSeqWriter* writer );



void   cvFlushSeqWriter( CvSeqWriter* writer );



void cvStartReadSeq( const CvSeq* seq, CvSeqReader* reader,
                               int reverse  );



int  cvGetSeqReaderPos( CvSeqReader* reader );



void   cvSetSeqReaderPos( CvSeqReader* reader, int index,
                                     int is_relative );


void*  cvCvtSeqToArray( CvSeq* seq, CvArr* arr,
                                   CvSlice slice );


CvSeq* cvMakeSeqHeaderForArray( int seq_type, int header_size,
                                           int elem_size, void* elements, int total,
                                           CvSeq* seq, CvSeqBlock* block );


CvSeq* cvSeqSlice( CvSeq* seq, CvSlice slice,
                             CvMemStorage* storage ,
                             int copy_data );

CvSeq* cvCloneSeq( CvSeq* seq, CvMemStorage* storage );

void  cvSeqRemoveSlice( CvSeq* seq, CvSlice slice );


void  cvSeqInsertSlice( CvSeq* seq, int index, const CvArr* from_arr );


typedef int (* CvCmpFunc)
                     (const void* a, const void* b, void* userdata );


void cvSeqSort( CvSeq* seq, CvCmpFunc func, void* userdata );


void cvSeqInvert( CvSeq* seq );


int  cvPartitionSeq( CvSeq* seq, CvMemStorage* storage, CvSeq** comps,
                                CvCmpFunc is_equal, void* userdata, int is_set );


void  cvChangeSeqBlock( CvSeqReader* reader, int direction );
void  cvCreateSeqBlock( CvSeqWriter* writer );



CvSet*  cvCreateSet( int set_flags, int header_size,
                                int elem_size, CvMemStorage* storage );


int  cvSetAdd( CvSet* set_header, CvSetElem* element ,
                          CvSetElem** inserted_element  );


CvSetElem* cvSetNew( CvSet* set_header );
void cvSetRemoveByPtr( CvSet* set_header, void* _elem );

void   cvSetRemove( CvSet* set_header, int index );



CvSetElem*  cvGetSetElem( CvSet* set_header, int index );



void  cvClearSet( CvSet* set_header );



CvGraph*   cvCreateGraph( int graph_flags, int header_size,
                                  int vtx_size, int edge_size,
                                  CvMemStorage* storage );


int  cvGraphAddVtx( CvGraph* graph, CvGraphVtx* vertex ,
                               CvGraphVtx** inserted_vertex  );



void   cvGraphRemoveVtx( CvGraph* graph, int index );
void   cvGraphRemoveVtxByPtr( CvGraph* graph, CvGraphVtx* vtx );



int  cvGraphAddEdge( CvGraph* graph,
                                int start_idx, int end_idx,
                                CvGraphEdge* edge ,
                                CvGraphEdge** inserted_edge  );

int  cvGraphAddEdgeByPtr( CvGraph* graph,
                               CvGraphVtx* start_vtx, CvGraphVtx* end_vtx,
                               CvGraphEdge* edge ,
                               CvGraphEdge** inserted_edge  );


void  cvGraphRemoveEdge( CvGraph* graph, int start_idx, int end_idx );
void  cvGraphRemoveEdgeByPtr( CvGraph* graph, CvGraphVtx* start_vtx,
                                         CvGraphVtx* end_vtx );


CvGraphEdge*  cvFindGraphEdge( CvGraph* graph, int start_idx, int end_idx );
CvGraphEdge*  cvFindGraphEdgeByPtr( CvGraph* graph, CvGraphVtx* start_vtx,
                                               CvGraphVtx* end_vtx );
#define cvGraphFindEdge cvFindGraphEdge
#define cvGraphFindEdgeByPtr cvFindGraphEdgeByPtr


void  cvClearGraph( CvGraph* graph );



int  cvGraphVtxDegree( CvGraph* graph, int vtx_idx );
int  cvGraphVtxDegreeByPtr( CvGraph* graph, CvGraphVtx* vtx );



#define cvGetGraphVtx( graph, idx ) (CvGraphVtx*)cvGetSetElem((CvSet*)(graph), (idx))


#define cvGraphVtxIdx( graph, vtx ) ((vtx)->flags & CV_SET_ELEM_IDX_MASK)


#define cvGraphEdgeIdx( graph, edge ) ((edge)->flags & CV_SET_ELEM_IDX_MASK)


#define  CV_GRAPH_VERTEX        1
#define  CV_GRAPH_TREE_EDGE     2
#define  CV_GRAPH_BACK_EDGE     4
#define  CV_GRAPH_FORWARD_EDGE  8
#define  CV_GRAPH_CROSS_EDGE    16
#define  CV_GRAPH_ANY_EDGE      30
#define  CV_GRAPH_NEW_TREE      32
#define  CV_GRAPH_BACKTRACKING  64
#define  CV_GRAPH_OVER          -1

#define  CV_GRAPH_ALL_ITEMS    -1


#define  CV_GRAPH_ITEM_VISITED_FLAG  (1 << 30)
#define  CV_IS_GRAPH_VERTEX_VISITED(vtx)     (((CvGraphVtx*)(vtx))->flags & CV_GRAPH_ITEM_VISITED_FLAG)
#define  CV_IS_GRAPH_EDGE_VISITED(edge)     (((CvGraphEdge*)(edge))->flags & CV_GRAPH_ITEM_VISITED_FLAG)
#define  CV_GRAPH_SEARCH_TREE_NODE_FLAG   (1 << 29)
#define  CV_GRAPH_FORWARD_EDGE_FLAG       (1 << 28)

typedef struct CvGraphScanner
{
    CvGraphVtx* vtx;
    CvGraphVtx* dst;
    CvGraphEdge* edge;

    CvGraph* graph;
    CvSeq*   stack;
    int      index;
    int      mask;
}
CvGraphScanner;


void  cvStartScanGraph( CvGraph* graph, CvGraphScanner* scanner,
                                  CvGraphVtx* vtx ,
                                  int mask );


void  cvEndScanGraph( CvGraphScanner* scanner );


int  cvNextGraphItem( CvGraphScanner* scanner );


CvGraph* cvCloneGraph( const CvGraph* graph, CvMemStorage* storage );




void cvLUT( const CvArr* srcarr, CvArr* dstarr, const CvArr* lutarr );



#define CV_BLUR_NO_SCALE 0
#define CV_BLUR  1
#define CV_GAUSSIAN  2
#define CV_MEDIAN 3
#define CV_BILATERAL 4

void cvSmooth( const CvArr* srcarr, CvArr* dstarr,
                          int smoothtype,
                          int param1 ,
                          int param2 );


void cvIntegral( const CvArr* image, CvArr* sumImage,
                           CvArr* sumSqImage ,
                           CvArr* tiltedSumImage );


void  cvPyrDown( const CvArr* src, CvArr* dst,
                            int filter );


void  cvPyrUp( const CvArr* src, CvArr* dst,
                          int filter );







void cvPyrSegmentation( IplImage* src,
                               IplImage* dst,
                               CvMemStorage *storage,
                               CvSeq **comp,
                               int level, double threshold1,
                               double threshold2 );


#define CV_SCHARR -1


void cvSobel( const CvArr* src, CvArr* dst,
                        int xorder, int yorder,
                        int apertureSize );


void cvLaplace( const CvArr* src, CvArr* dst,
                          int apertureSize  );


#define  CV_BGR2BGRA    0
#define  CV_RGB2RGBA    CV_BGR2BGRA

#define  CV_BGRA2BGR    1
#define  CV_RGBA2RGB    CV_BGRA2BGR

#define  CV_BGR2RGBA    2
#define  CV_RGB2BGRA    CV_BGR2RGBA

#define  CV_RGBA2BGR    3
#define  CV_BGRA2RGB    CV_RGBA2BGR

#define  CV_BGR2RGB     4
#define  CV_RGB2BGR     CV_BGR2RGB

#define  CV_BGRA2RGBA   5
#define  CV_RGBA2BGRA   CV_BGRA2RGBA

#define  CV_BGR2GRAY    6
#define  CV_RGB2GRAY    7
#define  CV_GRAY2BGR    8
#define  CV_GRAY2RGB    CV_GRAY2BGR
#define  CV_GRAY2BGRA   9
#define  CV_GRAY2RGBA   CV_GRAY2BGRA
#define  CV_BGRA2GRAY   10
#define  CV_RGBA2GRAY   11

#define  CV_BGR2BGR565  12
#define  CV_RGB2BGR565  13
#define  CV_BGR5652BGR  14
#define  CV_BGR5652RGB  15
#define  CV_BGRA2BGR565 16
#define  CV_RGBA2BGR565 17
#define  CV_BGR5652BGRA 18
#define  CV_BGR5652RGBA 19

#define  CV_GRAY2BGR565 20
#define  CV_BGR5652GRAY 21

#define  CV_BGR2XYZ     22
#define  CV_RGB2XYZ     23
#define  CV_XYZ2BGR     24
#define  CV_XYZ2RGB     25

#define  CV_BGR2YCrCb   26
#define  CV_RGB2YCrCb   27
#define  CV_YCrCb2BGR   28
#define  CV_YCrCb2RGB   29

#define  CV_BGR2HSV     30
#define  CV_RGB2HSV     31

#define  CV_BGR2Lab     34
#define  CV_RGB2Lab     35

#define  CV_BayerBG2BGR 40
#define  CV_BayerGB2BGR 41
#define  CV_BayerRG2BGR 42
#define  CV_BayerGR2BGR 43

#define  CV_BayerBG2RGB CV_BayerRG2BGR
#define  CV_BayerGB2RGB CV_BayerGR2BGR
#define  CV_BayerRG2RGB CV_BayerBG2BGR
#define  CV_BayerGR2RGB CV_BayerGB2BGR

#define  CV_COLORCVT_MAX  48


void  cvCvtColor( const CvArr* src, CvArr* dst, int colorCvtCode );


#define  CV_INTER_NN        0
#define  CV_INTER_LINEAR    1



void  cvResize( const CvArr* src, CvArr* dst,
                           int method );


#define  CV_SHAPE_RECT      0
#define  CV_SHAPE_CROSS     1
#define  CV_SHAPE_ELLIPSE   2
#define  CV_SHAPE_CUSTOM    100


IplConvKernel*  cvCreateStructuringElementEx(
            int  cols, int  rows, int  anchorX, int  anchorY,
            int shape, int* values  );


void  cvReleaseStructuringElement( IplConvKernel** element );



void  cvErode( const CvArr* src, CvArr* dst,
                          IplConvKernel* element ,
                          int iterations  );


void  cvDilate( const CvArr* src, CvArr* dst,
                           IplConvKernel* element ,
                           int iterations  );

#define CV_MOP_OPEN         2
#define CV_MOP_CLOSE        3
#define CV_MOP_GRADIENT     4
#define CV_MOP_TOPHAT       5
#define CV_MOP_BLACKHAT     6


void  cvMorphologyEx( const CvArr* src, CvArr* dst,
                                 CvArr* temp, IplConvKernel* element,
                                 int operation, int iterations  );



typedef struct CvMoments
{
    double  m00, m10, m01, m20, m11, m02, m30, m21, m12, m03;
    double  mu20, mu11, mu02, mu30, mu21, mu12, mu03;
    double  inv_sqrt_m00;
} CvMoments;


void cvMoments( const CvArr* arr, CvMoments* moments, int binary );


double  cvGetSpatialMoment( CvMoments* moments, int x_order, int y_order );
double  cvGetCentralMoment( CvMoments* moments, int x_order, int y_order );
double  cvGetNormalizedCentralMoment( CvMoments* moments,
                                                 int x_order, int y_order );


typedef struct CvHuMoments
{
    double hu1, hu2, hu3, hu4, hu5, hu6, hu7;
} CvHuMoments;


void cvGetHuMoments( CvMoments*  moments, CvHuMoments*  hu_moments );





#define CV_RGB( r, g, b )  (int)((uchar)(b) + ((uchar)(g) << 8) + ((uchar)(r) << 16))
#define CV_FILLED -1


void  cvLine( CvArr* arr, CvPoint pt1, CvPoint pt2,
                         double color, int thickness ,
                         int connectivity );


void  cvLineAA( CvArr* arr, CvPoint pt1, CvPoint pt2,
                           double color, int scale );


void  cvRectangle( CvArr* arr, CvPoint pt1, CvPoint pt2,
                              double color, int thickness );


void  cvCircle( CvArr* arr, CvPoint center, int radius,
                           double color, int thickness );


void  cvCircleAA( CvArr* arr, CvPoint center, int radius,
                             double color, int scale  );


void  cvEllipse( CvArr* arr, CvPoint center, CvSize axes,
                            double angle, double startAngle, double endAngle,
                            double color, int thickness );

void  cvEllipseBox( CvArr* arr, CvBox2D box,
                               double color, int thickness );

void  cvEllipseAA( CvArr* arr, CvPoint center, CvSize axes,
                              double angle, double startAngle,
                              double endAngle, double color,
                              int scale  );


void  cvFillConvexPoly( CvArr* arr, CvPoint* pts, int npts, double color );



void  cvFillPoly( CvArr* arr, CvPoint** pts,
                             int* npts, int contours, double color );


void  cvPolyLine( CvArr* arr, CvPoint** pts, int* npts, int contours,
                             int closed, double color,
                             int thickness ,
                             int connectivity );


void  cvPolyLineAA( CvArr* arr, CvPoint** pts, int* npts, int contours,
                               int closed, double color, int scale  );


#define CV_FONT_VECTOR0  0

typedef struct CvFont
{
    const int*  data;
    CvSize      size;
    int         italic_scale;
    int         thickness;
    int         dx;
} CvFont;


void  cvInitFont( CvFont* font, int font_face,
                             double hscale, double vscale,
                             double italic_scale ,
                             int thickness  );


void  cvPutText( CvArr* arr, const char* text, CvPoint org,
                            CvFont* font, double color );


void  cvGetTextSize( const char* text_string, CvFont* font,
                                CvSize* text_size, int* ymin );





typedef struct CvLineIterator
{
    uchar* ptr;
    int  err;
    int  plus_delta;
    int  minus_delta;
    int  plus_step;
    int  minus_step;
} CvLineIterator;


int  cvInitLineIterator( const CvArr* arr, CvPoint pt1, CvPoint pt2,
                                    CvLineIterator* lineIterator,
                                    int connectivity );


#define CV_NEXT_LINE_POINT( iterator ) {     int mask =  (iterator).err < 0 ? -1 : 0;     (iterator).err += (iterator).minus_delta + ((iterator).plus_delta & mask);     (iterator).ptr += (iterator).minus_step + ((iterator).plus_step & mask); }


int  cvSampleLine( const CvArr* arr, CvPoint pt1, CvPoint pt2, void* buffer,
                              int connectivity );


void  cvGetRectSubPix( const CvArr* src, CvArr* dst, CvPoint2D32f center );



void  cvGetQuadrangleSubPix( const CvArr* src, CvArr* dstarr,
                                        const CvArr* matrixarr,
                                        int fillOutliers ,
                                        CvScalar fillvalue );


#define  CV_TM_SQDIFF        0
#define  CV_TM_SQDIFF_NORMED 1
#define  CV_TM_CCORR         2
#define  CV_TM_CCORR_NORMED  3
#define  CV_TM_CCOEFF        4
#define  CV_TM_CCOEFF_NORMED 5


void  cvMatchTemplate( const CvArr* arr, const CvArr* templ,
                                  CvArr* result, int method );

typedef float ( * CvDistanceFunction)
                     ( const float* a, const float* b, void* user_param );


float  cvCalcEMD2( const CvArr* signature1,
                              const CvArr* signature2,
                              CvDisType dist_type,
                              CvDistanceFunction dist_func ,
                              const CvArr* cost_matrix ,
                              CvArr* flow ,
                              float* lower_bound ,
                              void* user_param );




typedef struct _CvContourScanner* CvContourScanner;

typedef enum CvContourRetrievalMode
{
    CV_RETR_EXTERNAL = 0,
    CV_RETR_LIST     = 1,
    CV_RETR_CCOMP    = 2,
    CV_RETR_TREE     = 3
}
CvContourRetrievalMode;

typedef enum CvChainApproxMethod
{
    CV_CHAIN_CODE             = 0,
    CV_CHAIN_APPROX_NONE      = 1,
    CV_CHAIN_APPROX_SIMPLE    = 2,
    CV_CHAIN_APPROX_TC89_L1   = 3,
    CV_CHAIN_APPROX_TC89_KCOS = 4,
    CV_LINK_RUNS              = 5
} CvChainApproxMethod;



int  cvFindContours( CvArr* arr, CvMemStorage* storage,
                           CvSeq**  firstContour,
                           int  headerSize,
                           CvContourRetrievalMode mode,
                           CvChainApproxMethod method );



CvContourScanner   cvStartFindContours( CvArr* arr, CvMemStorage* storage,
                                        int header_size, CvContourRetrievalMode mode,
                                        CvChainApproxMethod method );


CvSeq*  cvFindNextContour( CvContourScanner scanner );



void   cvSubstituteContour( CvContourScanner scanner, CvSeq* newContour );



CvSeq*  cvEndFindContours( CvContourScanner* scanner );


void  cvDrawContours( CvArr *img, CvSeq* contour,
                                double external_color, double hole_color,
                                int max_level, int thickness ,
                                int connectivity );


typedef struct CvTreeNodeIterator
{
    const void* node;
    int level;
    int maxLevel;
}
CvTreeNodeIterator;

void cvInitTreeNodeIterator( CvTreeNodeIterator* treeIterator,
                                   const void* first, int maxLevel );
void* cvNextTreeNode( CvTreeNodeIterator* treeIterator );
void* cvPrevTreeNode( CvTreeNodeIterator* treeIterator );


void cvInsertNodeIntoTree( void* node, void* parent, void* frame );


void cvRemoveNodeFromTree( void* node, void* frame );


CvSeq* cvTreeToNodeSeq( const void* first, int header_size,
                                  CvMemStorage* storage );


CvSeq* cvApproxChains( CvSeq* src_seq, CvMemStorage* storage,
                            CvChainApproxMethod method ,
                            double parameter ,
                            int  minimal_perimeter ,
                            int  recursive );



typedef struct CvChainPtReader
{
/*  CV_SEQ_READER_FIELDS() */
    int          header_size;
    CvSeq*       seq;
    CvSeqBlock*  block;
    char*        ptr;
    char*        block_min;
    char*        block_max;
    int          delta_index;
    char*        prev_elem;
    char      code;
    CvPoint  pt;
    char      deltas[8][2];
    int       reserved[2];
} CvChainPtReader;


void  cvStartReadChainPoints( CvChain* chain, CvChainPtReader* reader );


CvPoint   cvReadChainPoint( CvChainPtReader* reader );







void  cvAbsDiff( const CvArr* srcA, const CvArr* srcB, CvArr* dst );



void  cvAbsDiffS( const CvArr* src, CvArr* dst, CvScalar value );
#define cvAbs( src, dst ) cvAbsDiffS( (src), (dst), cvScalarAll(0))




void  cvCalcOpticalFlowLK( const CvArr* srcA, const CvArr* srcB,
                                      CvSize winSize, CvArr* velx, CvArr* vely );


void  cvCalcOpticalFlowBM( const CvArr* srcA, const CvArr* srcB,
                                      CvSize blockSize, CvSize shiftSize,
                                      CvSize maxRange, int usePrevious,
                                      CvArr* velx, CvArr* vely );


void  cvCalcOpticalFlowHS( const CvArr* srcA, const CvArr* srcB,
                                      int usePrevious, CvArr* velx, CvArr* vely,
                                      double _lambda, CvTermCriteria criteria );

#define  CV_LKFLOW_PYR_A_READY       1
#define  CV_LKFLOW_PYR_B_READY       2
#define  CV_LKFLOW_INITIAL_GUESSES   4


void  cvCalcOpticalFlowPyrLK( const CvArr*  imgA, const CvArr*  imgB,
                                         CvArr*  pyrA, CvArr*  pyrB,
                                         CvPoint2D32f* featuresA,
                                         CvPoint2D32f* featuresB,
                                         int       count,
                                         CvSize    winSize,
                                         int       level,
                                         char*     status,
                                         float*    error,
                                         CvTermCriteria criteria,
                                         int       flags );



void  cvCalcAffineFlowPyrLK( const CvArr*  imgA, const CvArr*  imgB,
                                        CvArr*  pyrA, CvArr*  pyrB,
                                        CvPoint2D32f* featuresA,
                                        CvPoint2D32f* featuresB,
                                        float*  matrices, int  count,
                                        CvSize  winSize, int  level,
                                        char*  status, float* error,
                                        CvTermCriteria criteria, int flags );






void    cvUpdateMotionHistory( const CvArr* silhouette, CvArr* mhi,
                                          double timestamp, double mhiDuration );


void    cvCalcMotionGradient( const CvArr* mhi, CvArr* mask, CvArr* orientation,
                                         double delta1, double delta2,
                                         int aperture_size );


double  cvCalcGlobalOrientation( const CvArr* orientation, const CvArr* mask,
                                            const CvArr* mhi, double curr_mhi_timestamp,
                                            double mhi_duration );


CvSeq*  cvSegmentMotion( const CvArr* mhi, CvArr* seg_mask,
                                    CvMemStorage* storage,
                                    double timestamp, double seg_thresh );




void  cvAcc( const CvArr* image, CvArr* sum,
                        const CvArr* mask  );


void  cvSquareAcc( const CvArr* image, CvArr* sqSum,
                              const CvArr* mask  );


void  cvMultiplyAcc( const CvArr* imgA, const CvArr* imgB, CvArr* acc,
                                const CvArr* mask  );


void  cvRunningAvg( const CvArr* imgY, CvArr* imgU, double alpha,
                               const CvArr* mask  );





int  cvCamShift( const CvArr* imgProb, CvRect  windowIn,
                           CvTermCriteria criteria, CvConnectedComp* out,
                           CvBox2D* box );


int  cvMeanShift( const CvArr* imgProb, CvRect  windowIn,
                            CvTermCriteria criteria, CvConnectedComp* out );

typedef struct CvConDensation
{
    int MP;
    int DP;
    float* DynamMatr;
    float* State;
    int SamplesNum;
    float** flSamples;
    float** flNewSamples;
    float* flConfidence;
    float* flCumulative;
    float* Temp;
    float* RandomSample;
    CvRandState* RandS;
} CvConDensation;


CvConDensation*  cvCreateConDensation( int DP, int MP, int SamplesNum);


void  cvReleaseConDensation( CvConDensation** ConDensation);


void  cvConDensUpdateByTime( CvConDensation* ConDens);


void  cvConDensInitSampleSet( CvConDensation* conDens, CvMat* lowerBound,CvMat* upperBound);


typedef struct CvKalman
{
    int MP;
    int DP;
    int CP;

    float* PosterState;
    float* PriorState;
    float* DynamMatr;
    float* MeasurementMatr;
    float* MNCovariance;
    float* PNCovariance;
    float* KalmGainMatr;
    float* PriorErrorCovariance;
    float* PosterErrorCovariance;
    float* Temp1;
    float* Temp2;

    CvMat* state_pre;
    CvMat* state_post;
    CvMat* transition_matrix;
    CvMat* control_matrix;
    CvMat* measurement_matrix;
    CvMat* process_noise_cov;
    CvMat* measurement_noise_cov;
    CvMat* error_cov_pre;
    CvMat* gain;
    CvMat* error_cov_post;
    CvMat* temp1;
    CvMat* temp2;
    CvMat* temp3;
    CvMat* temp4;
    CvMat* temp5;

}
CvKalman;


CvKalman* cvCreateKalman( int dynamParams, int measureParams,
                                    int controlParams );


void  cvReleaseKalman( CvKalman** Kalman);


const CvMat*  cvKalmanPredict( CvKalman* Kalman,
                                         const CvMat* control );


const CvMat*  cvKalmanCorrect( CvKalman* Kalman, const CvMat* measurement );





typedef long CvSubdiv2DEdge;

#define CV_SUBDIV2D_VIRTUAL_POINT_FLAG (1 << 30)

typedef struct CvQuadEdge2D
{
/*  CV_QUADEDGE2D_FIELDS() */
    int flags;
    struct CvSubdiv2DPoint* pt[4];
    CvSubdiv2DEdge  next[4];
}
CvQuadEdge2D;

typedef struct CvSubdiv2DPoint
{
/*  CV_SUBDIV2D_POINT_FIELDS() */
    int            flags;
    CvSubdiv2DEdge first;
    CvPoint2D32f   pt;
}
CvSubdiv2DPoint;

typedef struct CvSubdiv2D
{
/*  CV_SUBDIV2D_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvSetElem* free_elems;
    CvSet* edges;
    int  quad_edges;
    int  is_geometry_valid;
    CvSubdiv2DEdge recent_edge;
    CvPoint2D32f  topleft;
    CvPoint2D32f  bottomright;
}
CvSubdiv2D;


typedef enum CvSubdiv2DPointLocation
{
    CV_PTLOC_ERROR = -2,
    CV_PTLOC_OUTSIDE_RECT = -1,
    CV_PTLOC_INSIDE = 0,
    CV_PTLOC_VERTEX = 1,
    CV_PTLOC_ON_EDGE = 2
}
CvSubdiv2DPointLocation;

typedef enum CvNextEdgeType
{
    CV_NEXT_AROUND_ORG   = 0x00,
    CV_NEXT_AROUND_DST   = 0x22,
    CV_PREV_AROUND_ORG   = 0x11,
    CV_PREV_AROUND_DST   = 0x33,
    CV_NEXT_AROUND_LEFT  = 0x13,
    CV_NEXT_AROUND_RIGHT = 0x31,
    CV_PREV_AROUND_LEFT  = 0x20,
    CV_PREV_AROUND_RIGHT = 0x02
}
CvNextEdgeType;


#define  CV_SUBDIV2D_NEXT_EDGE( edge )  (((CvQuadEdge2D*)((edge) & ~3))->next[(edge)&3])



void  cvInitSubdivDelaunay2D( CvSubdiv2D* subdiv, CvRect rect );


CvSubdiv2D*  cvCreateSubdiv2D( int subdiv_type, int header_size,
                                       int vtx_size, int quadedge_size,
                                       CvMemStorage* storage );




CvSubdiv2D* cvCreateSubdivDelaunay2D( CvRect rect, CvMemStorage* storage );

CvSubdiv2DPoint*  cvSubdivDelaunay2DInsert( CvSubdiv2D* subdiv, CvPoint2D32f pt);


CvSubdiv2DPointLocation  cvSubdiv2DLocate(
                               CvSubdiv2D* subdiv, CvPoint2D32f pt,
                               CvSubdiv2DEdge *_edge,
                               CvSubdiv2DPoint** _point  );


void  cvCalcSubdivVoronoi2D( CvSubdiv2D* subdiv );



void  cvClearSubdivVoronoi2D( CvSubdiv2D* subdiv );



CvSubdiv2DPoint* cvFindNearestPoint2D( CvSubdiv2D* subdiv, CvPoint2D32f pt );




CvSubdiv2DEdge  cvSubdiv2DNextEdge( CvSubdiv2DEdge edge );
CvSubdiv2DEdge  cvSubdiv2DRotateEdge( CvSubdiv2DEdge edge, int rotate );
CvSubdiv2DEdge  cvSubdiv2DSymEdge( CvSubdiv2DEdge edge );
CvSubdiv2DEdge  cvSubdiv2DGetEdge( CvSubdiv2DEdge edge, CvNextEdgeType type );
CvSubdiv2DPoint*  cvSubdiv2DEdgeOrg( CvSubdiv2DEdge edge );
CvSubdiv2DPoint*  cvSubdiv2DEdgeDst( CvSubdiv2DEdge edge );
double  cvTriangleArea( CvPoint2D32f a, CvPoint2D32f b, CvPoint2D32f c );

#define CV_POLY_APPROX_DP 0


CvSeq*  cvApproxPoly( const void* src_seq,
                                 int header_size, CvMemStorage* storage,
                                 int method, double parameter,
                                 int parameter2 );


double  cvArcLength( const void* curve,
                                CvSlice slice ,
                                int is_closed );
#define cvContourPerimeter( contour ) cvArcLength( contour, CV_WHOLE_SEQ, 1 )


CvRect  cvBoundingRect( const void* points, int update  );


double  cvContourArea( const void* contour,
                                  CvSlice slice );


CvBox2D  cvMinAreaRect2( const void* points,
                                    CvMemStorage* storage );


void  cvMinEnclosingCircle( const void* points,
                                       CvPoint2D32f* center, float* radius );

#define CV_CONTOURS_MATCH_I1  1
#define CV_CONTOURS_MATCH_I2  2
#define CV_CONTOURS_MATCH_I3  3


double  cvMatchShapes( const void* contour1, const void* contour2,
                                  int method, double parameter );


typedef struct CvContourTree
{
/*  CV_SEQUENCE_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvPoint p1;
    CvPoint p2;
} CvContourTree;


CvContourTree*   cvCreateContourTree( CvSeq* contour, CvMemStorage* storage,
                                                 double threshold );


CvSeq*  cvContourFromContourTree( CvContourTree *tree,
                                          CvMemStorage* storage,
                                          CvTermCriteria criteria );


#define  CV_CONTOUR_TREES_MATCH_I1  1

double  cvMatchContourTrees( CvContourTree *tree1,
                                        CvContourTree *tree2,
                                        int method, double threshold );


void  cvCalcPGH( const CvSeq* contour, CvHistogram* hist );

#define CV_CLOCKWISE         1
#define CV_COUNTER_CLOCKWISE 2


CvSeq* cvConvexHull2( const CvArr* input,
                                void* hull_storage ,
                                int orientation,
                                int return_points );


int  cvCheckContourConvexity( const CvArr* contour );


typedef struct CvConvexityDefect
{
    CvPoint* start;
    CvPoint* end;
    CvPoint* depth_point;
    float depth;
} CvConvexityDefect;



CvSeq*  cvConvexityDefects( const CvArr* contour, const CvArr* convexhull,
                                      CvMemStorage* storage );


CvBox2D cvFitEllipse2( const CvArr* points );




CvHistogram*  cvCreateHist( int dims, int* sizes, int type,
                                       float** ranges ,
                                       int uniform );


void  cvSetHistBinRanges( CvHistogram* hist, float** ranges,
                                    int uniform );


CvHistogram*  cvMakeHistHeaderForArray(
                            int  dims, int* sizes, CvHistogram* hist,
                            float* data, float** ranges ,
                            int uniform );


void  cvReleaseHist( CvHistogram** hist );


void  cvClearHist( CvHistogram* hist );


void  cvGetMinMaxHistValue( const CvHistogram* hist,
                                    float* value_min, float* value_max,
                                    int* idx_min ,
                                    int* idx_max );



void  cvNormalizeHist( CvHistogram* hist, double factor );



void  cvThreshHist( CvHistogram* hist, double thresh );

#define CV_COMP_CORREL      0
#define CV_COMP_CHISQR      1
#define CV_COMP_INTERSECT   2


double  cvCompareHist( const CvHistogram* hist1,
                                  const CvHistogram* hist2,
                                  int method);


void  cvCopyHist( const CvHistogram* src, CvHistogram** dst );



void  cvCalcBayesianProb( CvHistogram** src, int number,
                                    CvHistogram** dst);


void  cvCalcArrHist( CvArr** arr, CvHistogram* hist,
                                int doNotClear ,
                                const CvArr* mask  );

void  cvCalcHist( IplImage** img, CvHistogram* hist,
                             int doNotClear ,
                             const CvArr* mask  );

void  cvCalcArrBackProject( CvArr** img, CvArr* dst,
                                       const CvHistogram* hist );
#define  cvCalcBackProject(img, dst, hist) cvCalcArrBackProject((CvArr**)img, dst, hist)



void  cvCalcArrBackProjectPatch( CvArr** img, CvArr* dst, CvSize range,
                                            CvHistogram* hist, int method,
                                            double normFactor );
#define  cvCalcBackProjectPatch( img, dst, range, hist, method, normFactor )      cvCalcArrBackProjectPatch( (CvArr**)img, dst, range, hist, method, normFactor )



void  cvCalcProbDensity( const CvHistogram* hist, const CvHistogram* hist_mask,
                                    CvHistogram* hist_dens, double scale );


#define  CV_VALUE  1
#define  CV_ARRAY  2

void  cvSnakeImage( const IplImage* src, CvPoint* points,
                            int  length, float* alpha,
                            float* beta, float* gamma,
                            int coeffUsage, CvSize  win,
                            CvTermCriteria criteria, int calcGradient );


void  cvCalcImageHomography(float *line, CvPoint3D32f* center,
                                     float* intrinsic, float* homography);

#define CV_DIST_MASK_3   3
#define CV_DIST_MASK_5   5


void  cvDistTransform( const CvArr* src, CvArr* dst,
                                  CvDisType disType,
                                  int maskSize ,
                                  const float* mask );



#define CV_THRESH_BINARY      0
#define CV_THRESH_BINARY_INV  1
#define CV_THRESH_TRUNC       2
#define CV_THRESH_TOZERO      3
#define CV_THRESH_TOZERO_INV  4


void  cvThreshold( const CvArr*  src, CvArr*  dst,
                              double  thresh, double  maxValue, int type );

#define CV_ADAPTIVE_THRESH_MEAN_C  0
#define CV_ADAPTIVE_THRESH_GAUSSIAN_C  1


void  cvAdaptiveThreshold( const CvArr* src, CvArr* dst, double maxValue,
                                      int method,
                                      int type,
                                      int blockSize ,
                                      double param1 );

#define CV_FLOODFILL_FIXED_RANGE (1 << 16)
#define CV_FLOODFILL_MASK_ONLY   (1 << 17)


void  cvFloodFill( CvArr* arr, CvPoint seedPoint,
                              double newVal, double loDiff ,
                              double upDiff ,
                              CvConnectedComp* comp ,
                              int flags ,
                              CvArr* mask );




void  cvCanny( const CvArr* src, CvArr* dst, double low_threshold,
                          double high_threshold, int  aperture_size  );


void cvPreCornerDetect( const CvArr* src, CvArr* dst,
                                  int aperture_size  );


void  cvCornerEigenValsAndVecs( const CvArr* src, CvArr* eigenvv,
                                          int blockSize,
                                          int aperture_size  );


void  cvCornerMinEigenVal( const CvArr* src, CvArr* eigenval,
                                     int blockSize, int aperture_size  );


void  cvFindCornerSubPix( const CvArr* src,CvPoint2D32f*  corners,
                                     int count, CvSize win,CvSize zero_zone,
                                     CvTermCriteria  criteria );


void  cvGoodFeaturesToTrack( const CvArr* image, CvArr* eig_image,
                                       CvArr* temp_image, CvPoint2D32f* corners,
                                       int* corner_count, double  quality_level,
                                       double  min_distance,
                                       const CvArr* mask );

#define CV_HOUGH_STANDARD 0
#define CV_HOUGH_PROBABILISTIC 1
#define CV_HOUGH_MULTI_SCALE 2


CvSeq*  cvHoughLines2( CvArr* image, void* line_storage, int method,
                                  double rho, double theta, int threshold,
                                  double param1 , double param2 );


void  cvProject3D( CvPoint3D32f* points3D, int count,
                              CvPoint2D32f* points2D, int xIndx, int yIndx );


void  cvFitLine( const CvArr* points, CvDisType dist, double param,
                            double reps, double aeps, float* line );


#define CV_EIGOBJ_NO_CALLBACK     0
#define CV_EIGOBJ_INPUT_CALLBACK  1
#define CV_EIGOBJ_OUTPUT_CALLBACK 2
#define CV_EIGOBJ_BOTH_CALLBACK   3


typedef CvStatus ( * CvCallback)
                    (int index, void* buffer, void* userData);

typedef union
{
    CvCallback callback;
    void* data;
}
CvInput;


void  cvCalcCovarMatrixEx( int nObjects, void* input, int ioFlags,
                                      int ioBufSize, uchar* buffer, void* userData,
                                      IplImage* avg, float* covarMatrix );


void  cvCalcEigenObjects( int nObjects, void* input, void* output,
                                    int ioFlags, int ioBufSize, void* userData,
                                    CvTermCriteria* calcLimit, IplImage* avg,
                                    float* eigVals );


double  cvCalcDecompCoeff( IplImage* obj, IplImage* eigObj, IplImage* avg );


void  cvEigenDecomposite( IplImage* obj, int nEigObjs, void* eigInput,
                                    int ioFlags, void* userData, IplImage* avg,
                                    float* coeffs );


void  cvEigenProjection( void* eigInput, int nEigObjs, int ioFlags,
                                   void* userData, float* coeffs, IplImage* avg,
                                   IplImage* proj );


typedef struct CvEHMMState
{
    int num_mix;
    float* mu;
    float* inv_var;
    float* log_var_val;
    float* weight;

} CvEHMMState;

typedef union {
    CvEHMMState* state;
    struct CvEHMM* ehmm;
} CvEHMM_u;

typedef struct CvEHMM
{
    int level;
    int num_states;
    float*  transP;
    float** obsProb;
    CvEHMM_u u;
/*    union
    {
        CvEHMMState* state;
        struct CvEHMM* ehmm;
    } u; */
} CvEHMM;


CvEHMM*  cvCreate2DHMM( int* stateNumber, int* numMix, int obsSize );



void  cvRelease2DHMM( CvEHMM** hmm );



#define CV_COUNT_OBS(roi, win, delta, numObs ) {    (numObs)->width  =((roi)->width  -(win)->width  +(delta)->width)/(delta)->width;    (numObs)->height =((roi)->height -(win)->height +(delta)->height)/(delta)->height; }

typedef struct CvImgObsInfo
{
    int obs_x;
    int obs_y;
    int obs_size;
    float* obs;

    int* state;
    int* mix;

} CvImgObsInfo;


CvImgObsInfo*  cvCreateObsInfo( CvSize numObs, int obsSize );


void  cvReleaseObsInfo( CvImgObsInfo** obs_info );



void  cvImgToObs_DCT( const CvArr* arr, float* obs, CvSize dctSize,
                                 CvSize obsSize, CvSize delta );



void  cvUniformImgSegm( CvImgObsInfo* obs_info, CvEHMM* ehmm );


void  cvInitMixSegm( CvImgObsInfo** obs_info_array,
                               int num_img, CvEHMM* hmm );


void  cvEstimateHMMStateParams( CvImgObsInfo** obs_info_array,
                                        int num_img, CvEHMM* hmm );


void  cvEstimateTransProb( CvImgObsInfo** obs_info_array,
                                   int num_img, CvEHMM* hmm );


void  cvEstimateObsProb( CvImgObsInfo* obs_info,
                                   CvEHMM* hmm );


float  cvEViterbi( CvImgObsInfo* obs_info, CvEHMM* hmm );



void  cvMixSegmL2( CvImgObsInfo** obs_info_array,
                             int num_img, CvEHMM* hmm );


void  cvKMeans2( const CvArr* samples, int cluster_count,
                            CvArr* cluster_idx, CvTermCriteria termcrit );




void  cvUnDistortOnce( const CvArr* srcImage, CvArr* dstImage,
                                  const float* intrMatrix,
                                  const float* distCoeffs,
                                  int interpolate  );


void  cvUnDistortInit( const CvArr* srcImage, CvArr* undistMap,
                                  const float* intrMatrix,
                                  const float* distCoeffs,
                                  int interpolate  );


void  cvUnDistort( const CvArr* srcImage, CvArr* dstImage,
                              const CvArr* undistMap, int interpolate );
#define cvRemap cvUnDistort



void  cvConvertMap( const CvArr* srcImage, const CvArr* flUndistMap,
                               CvArr* undistMap, int iterpolate  );


void  cvCalibrateCamera( int           numImages,
                                    int*          numPoints,
                                    CvSize        imageSize,
                                    CvPoint2D32f* imagePoints32f,
                                    CvPoint3D32f* objectPoints32f,
                                    CvVect32f     distortion32f,
                                    CvMatr32f     cameraMatrix32f,
                                    CvVect32f     transVects32f,
                                    CvMatr32f     rotMatrs32f,
                                    int           useIntrinsicGuess);


void  cvCalibrateCamera_64d( int           numImages,
                                       int*          numPoints,
                                       CvSize        imageSize,
                                       CvPoint2D64d* imagePoints,
                                       CvPoint3D64d* objectPoints,
                                       CvVect64d     distortion,
                                       CvMatr64d     cameraMatrix,
                                       CvVect64d     transVects,
                                       CvMatr64d     rotMatrs,
                                       int           useIntrinsicGuess );


void  cvFindExtrinsicCameraParams( int           numPoints,
                                             CvSize        imageSize,
                                             CvPoint2D32f* imagePoints32f,
                                             CvPoint3D32f* objectPoints32f,
                                             CvVect32f     focalLength32f,
                                             CvPoint2D32f  principalPoint32f,
                                             CvVect32f     distortion32f,
                                             CvVect32f     rotVect32f,
                                             CvVect32f     transVect32f);


void  cvFindExtrinsicCameraParams_64d( int           numPoints,
                                                 CvSize        imageSize,
                                                 CvPoint2D64d* imagePoints,
                                                 CvPoint3D64d* objectPoints,
                                                 CvVect64d     focalLength,
                                                 CvPoint2D64d  principalPoint,
                                                 CvVect64d     distortion,
                                                 CvVect64d     rotVect,
                                                 CvVect64d     transVect);



#define CV_RODRIGUES_M2V  0
#define CV_RODRIGUES_V2M  1


void  cvRodrigues( CvMat* rotMatrix, CvMat* rotVector,
                              CvMat* jacobian, int convType);


void  cvProjectPoints( int             numPoints,
                                 CvPoint3D64d*   objectPoints,
                                 CvVect64d       rotVect,
                                 CvVect64d       transVect,
                                 CvVect64d       focalLength,
                                 CvPoint2D64d    principalPoint,
                                 CvVect64d       distortion,
                                 CvPoint2D64d*   imagePoints,
                                 CvVect64d       derivPointsRot,
                                 CvVect64d       derivPointsTrans,
                                 CvVect64d       derivPointsFocal,
                                 CvVect64d       derivPointsPrincipal,
                                 CvVect64d       derivPointsDistort);


void cvProjectPointsSimple(  int numPoints,
                                    CvPoint3D64d * objectPoints,
                                    CvVect64d rotMatr,
                                    CvVect64d transVect,
                                    CvMatr64d cameraMatrix,
                                    CvVect64d distortion,
                                    CvPoint2D64d* imagePoints);


int  cvFindChessBoardCornerGuesses( const CvArr* arr, CvArr* thresh,
                                               CvMemStorage* storage, CvSize etalon_size,
                                               CvPoint2D32f* corners,
                                               int *corner_count );


typedef struct CvPOSITObject CvPOSITObject;


CvPOSITObject*  cvCreatePOSITObject( CvPoint3D32f* points, int numPoints );



void  cvPOSIT(  CvPOSITObject* pObject, CvPoint2D32f* imagePoints,
                           double focalLength, CvTermCriteria criteria,
                           CvMatr32f rotation, CvVect32f translation);


void  cvReleasePOSITObject( CvPOSITObject**  ppObject );



void cvMake2DPoints(CvMat* srcPoint,CvMat* dstPoint);
void cvMake3DPoints(CvMat* srcPoint,CvMat* dstPoint);
int cvSolveCubic(CvMat* coeffs,CvMat* result);


#define CV_FM_7POINT 1
#define CV_FM_8POINT 2
#define CV_FM_RANSAC 3
#define CV_FM_LMEDS  4
int cvFindFundamentalMat( CvMat* points1, CvMat* points2,
                                     CvMat* fundMatr, int method,
                                     double param1, double param2,
                                     CvMat* status );


void cvComputeCorrespondEpilines(CvMat* points,
                                            int pointImageID,
                                            CvMat* fundMatr,
                                            CvMat* corrLines);






void  cvMakeScanlines( const CvMatrix3* matrix,
                                CvSize     imgSize,
                                int*       scanlines_1,
                                int*       scanlines_2,
                                int*       lens_1,
                                int*       lens_2,
                                int*       numlines);


void  cvPreWarpImage( int       numLines,
                               IplImage* img,
                               uchar*    dst,
                               int*      dst_nums,
                               int*      scanlines);


void  cvFindRuns( int    numLines,
                           uchar* prewarp_1,
                           uchar* prewarp_2,
                           int*   line_lens_1,
                           int*   line_lens_2,
                           int*   runs_1,
                           int*   runs_2,
                           int*   num_runs_1,
                           int*   num_runs_2);


void  cvDynamicCorrespondMulti( int  lines,
                                         int* first,
                                         int* first_runs,
                                         int* second,
                                         int* second_runs,
                                         int* first_corr,
                                         int* second_corr);


void  cvMakeAlphaScanlines( int*  scanlines_1,
                                     int*  scanlines_2,
                                     int*  scanlines_a,
                                     int*  lens,
                                     int   numlines,
                                     float alpha);


void  cvMorphEpilinesMulti( int    lines,
                                     uchar* first_pix,
                                     int*   first_num,
                                     uchar* second_pix,
                                     int*   second_num,
                                     uchar* dst_pix,
                                     int*   dst_num,
                                     float  alpha,
                                     int*   first,
                                     int*   first_runs,
                                     int*   second,
                                     int*   second_runs,
                                     int*   first_corr,
                                     int*   second_corr);


void  cvPostWarpImage( int       numLines,
                                uchar*    src,
                                int*      src_nums,
                                IplImage* img,
                                int*      scanlines);


void  cvDeleteMoire( IplImage*  img);




int  cvLoadPrimitives( const char* proc_type  );


int  cvFillInternalFuncsTable(void* table);


void  cvGetLibraryInfo( const char** version, int* loaded,
                                   const char** loaded_modules );


CVStatus cvGetErrStatus( void );


void cvSetErrStatus( CVStatus status );



int  cvGetErrMode( void );


void cvSetErrMode( int mode );


CVStatus cvError( CVStatus code, const char *func,
                         const char *context, const char *file, int line);


const char* cvErrorStr( CVStatus status );



CVErrorCallBack cvRedirectError(CVErrorCallBack cvErrorFunc);



CVStatus cvNulDevReport( CVStatus status, const char *funcName,
                                const char *context, const char *file, int line );

CVStatus cvStdErrReport( CVStatus status, const char *funcName,
                                const char *context, const char *file, int line );

CVStatus cvGuiBoxReport( CVStatus status, const char *funcName,
                                const char *context, const char *file, int line);


void cvGetCallStack(CvStackRecord** stack, int* size);


void cvStartProfile( const char* call, const char* file, int line );


void cvEndProfile( const char* file, int line );

typedef void (* CvStartProfileFunc)(const char*,const char*,int);
typedef void (* CvEndProfileFunc)(const char*,int);


void cvSetProfile( CvStartProfileFunc startProfile,
                             CvEndProfileFunc endProfile );

void cvRemoveProfile();


typedef void* ( *CvAllocFunc)(size_t, const char*, int);
typedef int ( *CvFreeFunc)(void**, const char*, int);


void cvSetMemoryManager( CvAllocFunc allocFunc ,
                                   CvFreeFunc freeFunc );


typedef IplImage* (* Cv_iplCreateImageHeader)
                            (int,int,int,char*,char*,int,int,int,int,int,
                            IplROI*,IplImage*,void*,IplTileInfo*);
typedef void (* Cv_iplAllocateImageData)(IplImage*,int,int);

typedef void (* Cv_iplDeallocate)(IplImage*,int);

typedef IplROI* (* Cv_iplCreateROI)(int,int,int,int,int);

typedef IplImage* (* Cv_iplCloneImage)(const IplImage*);



void
cvSetIPLAllocators( Cv_iplCreateImageHeader createHeader,
                    Cv_iplAllocateImageData allocateData,
                    Cv_iplDeallocate deallocate,
                    Cv_iplCreateROI createROI,
                    Cv_iplCloneImage cloneImage );

#define CV_TURN_ON_IPL_COMPATIBILITY()     cvSetIPLAllocators( iplCreateImageHeader, iplAllocateImage, iplDeallocate, iplCreateROI, iplCloneImage )






typedef struct CvFileStorage CvFileStorage;


#define CV_STORAGE_READ          0
#define CV_STORAGE_WRITE_TEXT    1
#define CV_STORAGE_WRITE_BINARY  2


#define CV_WRITE_TREE      2


CvFileStorage*  cvOpenFileStorage( const char* filename,
                                              CvMemStorage* storage,
                                              int flags );


void cvReleaseFileStorage( CvFileStorage** storage );


typedef struct CvAttrList
{
    char** attr;
    struct CvAttrList* next;
}
CvAttrList;

CvAttrList cvAttrList( char** attr ,
                                 CvAttrList* next  );

const char* cvAttrValue( const CvAttrList* attr, const char* attr_name );

struct CvTypeInfo;

typedef struct CvFileNode
{
/*  CV_TREE_NODE_FIELDS(CvFileNode) */
    int       flags;
    int       header_size;
    struct    CvFileNode* h_prev;
    struct    CvFileNode* h_next;
    struct    CvFileNode* v_prev;
    struct    CvFileNode* v_next;
    const char* tagname;
    const char* name;
    CvAttrList* attr;
    struct CvFileNode* hash_next;
    unsigned hash_val;
    int elem_size;
    struct CvTypeInfo* typeinfo;
    const char* body;
    const void* content;
}
CvFileNode;



void cvWrite( CvFileStorage* storage, const char* name,
                         const void* struct_ptr,
                         CvAttrList attributes,
                         int flags );


void cvStartWriteStruct( CvFileStorage* storage, const char* name,
                                    const char* type_name ,
                                    const void* struct_ptr ,
                                    CvAttrList attributes );


void cvEndWriteStruct( CvFileStorage* storage );


void cvWriteElem( CvFileStorage* storage,
                             const char* name,
                             const char* elem_spec,
                             const void* data_ptr );


CvFileNode* cvGetFileNode( CvFileStorage* storage, const char* name );


const void* cvReadFileNode( CvFileStorage* storage, CvFileNode* node,
                                       CvAttrList** attr_list );

const void* cvRead( CvFileStorage* storage, const char* name,
                               CvAttrList** attr_list );

int cvReadElem( CvFileStorage* storage, const char* name, void* data_ptr );



typedef int ( *CvIsInstanceFunc)(const void* struct_ptr);
typedef void ( *CvReleaseFunc)(void** struct_dblptr);
typedef void* ( *CvReadFunc)( CvFileStorage* storage,
                                                  CvFileNode* node );
typedef void ( *CvWriteFunc)( CvFileStorage* storage,
                                                  const char* name,
                                                  const void* struct_ptr,
                                                  CvAttrList attributes,
                                                  int flags );
typedef void* ( *CvCloneFunc)( const void* struct_ptr);

typedef struct CvTypeInfo
{
    int flags;
    int header_size;
    struct CvTypeInfo* prev;
    struct CvTypeInfo* next;
    const char* type_name;
    CvIsInstanceFunc is_instance;
    CvReleaseFunc release;
    CvReadFunc read;
    CvWriteFunc write;
    CvCloneFunc clone;
}
CvTypeInfo;

CvTypeInfo* cvRegisterType( CvTypeInfo* info_data );
void        cvUnregisterType( const char* type_name );
CvTypeInfo* cvFirstType(void);
CvTypeInfo* cvFindType( const char* type_name );
CvTypeInfo* cvTypeOf( const void* struct_ptr );


void cvRelease( void** struct_ptr );
void* cvClone( const void* struct_ptr );
